##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2013-2016 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from common.srdhelper import bitpack

'''
OUTPUT_PYTHON format:

Packet:
[<ptype>, <pdata>]

<ptype>, <pdata>
 - 'ITEM', [<item>, <itembitsize>]
 - 'WORD', [<word>, <wordbitsize>, <worditemcount>]

<item>:
 - A single item (a number). It can be of arbitrary size. The max. number
   of bits in this item is specified in <itembitsize>.

<itembitsize>:
 - The size of an item (in bits). For a 4-bit parallel bus this is 4,
   for a 16-bit parallel bus this is 16, and so on.

<word>:
 - A single word (a number). It can be of arbitrary size. The max. number
   of bits in this word is specified in <wordbitsize>. The (exact) number
   of items in this word is specified in <worditemcount>.

<wordbitsize>:
 - The size of a word (in bits). For a 2-item word with 8-bit items
   <wordbitsize> is 16, for a 3-item word with 4-bit items <wordbitsize>
   is 12, and so on.

<worditemcount>:
 - The size of a word (in number of items). For a 4-item word (no matter
   how many bits each item consists of) <worditemcount> is 4, for a 7-item
   word <worditemcount> is 7, and so on.
'''

def channel_list(num_channels):
    l = [{'id': 'clk', 'name': 'CLK', 'desc': 'Clock line'}]
    for i in range(num_channels):
        d = {'id': 'd%d' % i, 'name': 'D%d' % i, 'desc': 'Data line %d' % i}
        l.append(d)
    return tuple(l)

class ChannelError(Exception):
    pass

NUM_CHANNELS = 8

class Decoder(srd.Decoder):
    api_version = 3
    id = 'parallel'
    name = 'Parallel'
    longname = 'Parallel sync bus'
    desc = 'Generic parallel synchronous bus.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = ['parallel']
    tags = ['Util']
    optional_channels = channel_list(NUM_CHANNELS)
    options = (
        {'id': 'clock_edge', 'desc': 'Clock edge to sample on',
            'default': 'rising', 'values': ('rising', 'falling')},
        {'id': 'wordsize', 'desc': 'Data wordsize (# bus cycles)',
            'default': 0},
        {'id': 'endianness', 'desc': 'Data endianness',
            'default': 'little', 'values': ('little', 'big')},
    )
    annotations = (
        ('items', 'Items'),
        ('words', 'Words'),
    )
    annotation_rows = (
        ('items', 'Items', (0,)),
        ('words', 'Words', (1,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.items = []
        self.saved_item = None
        self.ss_item = self.es_item = None
        self.saved_word = None
        self.ss_word = self.es_word = None
        self.first = True

    def start(self):
        self.out_python = self.register(srd.OUTPUT_PYTHON)
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putpb(self, data):
        self.put(self.ss_item, self.es_item, self.out_python, data)

    def putb(self, data):
        self.put(self.ss_item, self.es_item, self.out_ann, data)

    def putpw(self, data):
        self.put(self.ss_word, self.es_word, self.out_python, data)

    def putw(self, data):
        self.put(self.ss_word, self.es_word, self.out_ann, data)

    def handle_bits(self, item, used_pins):

        # If a word was previously accumulated, then emit its annotation
        # now after its end samplenumber became available.
        if self.saved_word is not None:
            if self.options['wordsize'] > 0:
                self.es_word = self.samplenum
                self.putw([1, [self.fmt_word.format(self.saved_word)]])
                self.putpw(['WORD', self.saved_word])
            self.saved_word = None

        # Defer annotations for individual items until the next sample
        # is taken, and the previous sample's end samplenumber has
        # become available.
        if self.first:
            # Save the start sample and item for later (no output yet).
            self.ss_item = self.samplenum
            self.first = False
            self.saved_item = item
        else:
            # Output the saved item (from the last CLK edge to the current).
            self.es_item = self.samplenum
            self.putpb(['ITEM', self.saved_item])
            self.putb([0, [self.fmt_item.format(self.saved_item)]])
            self.ss_item = self.samplenum
            self.saved_item = item

        # Get as many items as the configured wordsize specifies.
        if not self.items:
            self.ss_word = self.samplenum
        self.items.append(item)
        ws = self.options['wordsize']
        if len(self.items) < ws:
            return

        # Collect words and prepare annotation details, but defer emission
        # until the end samplenumber becomes available.
        endian = self.options['endianness']
        if endian == 'big':
            self.items.reverse()
        word = sum([self.items[i] << (i * used_pins) for i in range(ws)])
        self.saved_word = word
        self.items = []

    def decode(self):
        # Determine which (optional) channels have input data. Insist in
        # a non-empty input data set. Cope with sparse connection maps.
        # Store enough state to later "compress" sampled input data.
        max_possible = len(self.optional_channels)
        idx_channels = [
            idx if self.has_channel(idx) else None
            for idx in range(max_possible)
        ]
        has_channels = [idx for idx in idx_channels if idx is not None]
        if not has_channels:
            raise ChannelError('At least one channel has to be supplied.')
        max_connected = max(has_channels)

        # Determine .wait() conditions, depending on the presence of a
        # clock signal. Either inspect samples on the configured edge of
        # the clock, or inspect samples upon ANY edge of ANY of the pins
        # which provide input data.
        if self.has_channel(0):
            edge = self.options['clock_edge'][0]
            conds = {0: edge}
        else:
            conds = [{idx: 'e'} for idx in has_channels]

        # Pre-determine which input data to strip off, the width of
        # individual items and multiplexed words, as well as format
        # strings here. This simplifies call sites which run in tight
        # loops later.
        idx_strip = max_connected + 1
        num_item_bits = idx_strip - 1
        num_word_items = self.options['wordsize']
        num_word_bits = num_item_bits * num_word_items
        num_digits = (num_item_bits + 3) // 4
        self.fmt_item = "{{:0{}x}}".format(num_digits)
        num_digits = (num_word_bits + 3) // 4
        self.fmt_word = "{{:0{}x}}".format(num_digits)

        # Keep processing the input stream. Assume "always zero" for
        # not-connected input lines. Pass data bits (all inputs except
        # clock) to the handle_bits() method.
        while True:
            pins = self.wait(conds)
            bits = [0 if idx is None else pins[idx] for idx in idx_channels]
            item = bitpack(bits[1:idx_strip])
            self.handle_bits(item, num_item_bits)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Karl Palsson <karlp@tweak.net.au>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from .lists import *

TX, RX = range(2)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'mrf24j40'
    name = 'MRF24J40'
    longname = 'Microchip MRF24J40'
    desc = 'IEEE 802.15.4 2.4 GHz RF tranceiver chip.'
    license = 'gplv2+'
    inputs = ['spi']
    outputs = []
    tags = ['IC', 'Wireless/RF']
    annotations = (
        ('sread', 'Short register read commands'),
        ('swrite', 'Short register write commands'),
        ('lread', 'Long register read commands'),
        ('lwrite', 'Long register write commands'),
        ('warning', 'Warnings'),
        ('tx-frame', 'TX frame'),
        ('rx-frame', 'RX frame'),
        ('tx-retry-1', '1x TX retry'),
        ('tx-retry-2', '2x TX retry'),
        ('tx-retry-3', '3x TX retry'),
        ('tx-fail', 'TX fail (too many retries)'),
        ('ccafail', 'CCAFAIL (channel busy)'),
    )
    annotation_rows = (
        ('read', 'Read', (0, 2)),
        ('write', 'Write', (1, 3)),
        ('warnings', 'Warnings', (4,)),
        ('tx-frames', 'TX frames', (5,)),
        ('rx-frames', 'RX frames', (6,)),
        ('tx-retries-1', '1x TX retries', (7,)),
        ('tx-retries-2', '2x TX retries', (8,)),
        ('tx-retries-3', '3x TX retries', (9,)),
        ('tx-fails', 'TX fails', (10,)),
        ('ccafails', 'CCAFAILs', (11,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.ss_cmd, self.es_cmd = 0, 0
        self.ss_frame, self.es_frame = [0, 0], [0, 0]
        self.mosi_bytes, self.miso_bytes = [], []
        self.framecache = [[], []]

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss_cmd, self.es_cmd, self.out_ann, data)

    def putw(self, pos, msg):
        self.put(pos[0], pos[1], self.out_ann, [4, [msg]])

    def reset_data(self):
        self.mosi_bytes = []
        self.miso_bytes = []

    def handle_short(self):
        write = self.mosi_bytes[0] & 0x1
        reg = (self.mosi_bytes[0] >> 1) & 0x3f
        reg_desc = sregs.get(reg, 'illegal')
        for rxtx in (RX, TX):
            if self.framecache[rxtx] == []:
                continue
            bit0 = self.mosi_bytes[1] & (1 << 0)
            if rxtx == TX and not (reg_desc == 'TXNCON' and bit0 == 1):
                continue
            if rxtx == RX and not (reg_desc == 'RXFLUSH' and bit0 == 1):
                continue
            idx = 5 if rxtx == TX else 6
            xmitdir = 'TX' if rxtx == TX else 'RX'
            frame = ' '.join(['%02X' % b for b in self.framecache[rxtx]])
            self.put(self.ss_frame[rxtx], self.es_frame[rxtx], self.out_ann,
                [idx, ['%s frame: %s' % (xmitdir, frame)]])
            self.framecache[rxtx] = []
        if write:
            self.putx([1, ['%s: %#x' % (reg_desc, self.mosi_bytes[1])]])
        else:
            self.putx([0, ['%s: %#x' % (reg_desc, self.miso_bytes[1])]])
            numretries = (self.miso_bytes[1] & 0xc0) >> 6
            if reg_desc == 'TXSTAT' and numretries > 0:
                txfail = 1 if ((self.miso_bytes[1] & (1 << 0)) != 0) else 0
                idx = 6 + numretries + txfail
                if txfail:
                    self.putx([idx, ['TX fail (>= 4 retries)', 'TX fail']])
                else:
                    self.putx([idx, ['TX retries: %d' % numretries]])
            if reg_desc == 'TXSTAT' and (self.miso_bytes[1] & (1 << 5)) != 0:
                self.putx([11, ['CCAFAIL (channel busy)', 'CCAFAIL']])

    def handle_long(self):
        dword = self.mosi_bytes[0] << 8 | self.mosi_bytes[1]
        write = dword & (0x1 << 4)
        reg = dword >> 5 & 0x3ff
        if reg >= 0x0:
            reg_desc = 'TX:%#x' % reg
        if reg >= 0x80:
            reg_desc = 'TX beacon:%#x' % reg
        if reg >= 0x100:
            reg_desc = 'TX GTS1:%#x' % reg
        if reg >= 0x180:
            reg_desc = 'TX GTS2:%#x' % reg
        if reg >= 0x200:
            reg_desc = lregs.get(reg, 'illegal')
        if reg >= 0x280:
            reg_desc = 'Security keys:%#x' % reg
        if reg >= 0x2c0:
            reg_desc = 'Reserved:%#x' % reg
        if reg >= 0x300:
            reg_desc = 'RX:%#x' % reg

        if write:
            self.putx([3, ['%s: %#x' % (reg_desc, self.mosi_bytes[2])]])
        else:
            self.putx([2, ['%s: %#x' % (reg_desc, self.miso_bytes[2])]])

        for rxtx in (RX, TX):
            if rxtx == RX and reg_desc[:3] != 'RX:':
                continue
            if rxtx == TX and reg_desc[:3] != 'TX:':
                continue
            if len(self.framecache[rxtx]) == 0:
                self.ss_frame[rxtx] = self.ss_cmd
            self.es_frame[rxtx] = self.es_cmd
            self.framecache[rxtx] += [self.mosi_bytes[2]] if rxtx == TX else [self.miso_bytes[2]]

    def decode(self, ss, es, data):
        ptype = data[0]
        if ptype == 'CS-CHANGE':
            # If we transition high mid-stream, toss out our data and restart.
            cs_old, cs_new = data[1:]
            if cs_old is not None and cs_old == 0 and cs_new == 1:
                if len(self.mosi_bytes) not in (0, 2, 3):
                    self.putw([self.ss_cmd, es], 'Misplaced CS!')
                    self.reset_data()
            return

        # Don't care about anything else.
        if ptype != 'DATA':
            return
        mosi, miso = data[1:]

        self.ss, self.es = ss, es

        if len(self.mosi_bytes) == 0:
            self.ss_cmd = ss
        self.mosi_bytes.append(mosi)
        self.miso_bytes.append(miso)

        # Everything is either 2 bytes or 3 bytes.
        if len(self.mosi_bytes) < 2:
            return

        if self.mosi_bytes[0] & 0x80:
            if len(self.mosi_bytes) == 3:
                self.es_cmd = es
                self.handle_long()
                self.reset_data()
        else:
            self.es_cmd = es
            self.handle_short()
            self.reset_data()



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# Addresses/devices. Items that are not listed are reserved/unknown.
address = {
    0x40: 'Matsui TV',
}

digits = {
    0: ['0', '0'],
    1: ['1', '1'],
    2: ['2', '2'],
    3: ['3', '3'],
    4: ['4', '4'],
    5: ['5', '5'],
    6: ['6', '6'],
    7: ['7', '7'],
    8: ['8', '8'],
    9: ['9', '9'],
}

# Commands. Items that are not listed are reserved/unknown.
command = {
    0x40: dict(list(digits.items()) + list({
        11: ['-/--', '-/--'],
        16: ['Mute', 'M'],
        18: ['Standby', 'StBy'],
        26: ['Volume up', 'Vol+'],
        27: ['Program up', 'P+'],
        30: ['Volume down', 'Vol-'],
        31: ['Program down', 'P-'],
        68: ['AV', 'AV'],
    }.items())),
}



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2016 Rudolf Reuter <reuterru@arcor.de>
## Copyright (C) 2017 Marcus Comstedt <marcus@mc.pp.se>
## Copyright (C) 2019 Gerhard Sittig <gerhard.sittig@gmx.net>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# This file was created from earlier implementations of the 'gpib' and
# the 'iec' protocol decoders. It combines the parallel and the serial
# transmission variants in a single instance with optional inputs for
# maximum code re-use.

# TODO
# - Extend annotations for improved usability.
#   - Keep talkers' data streams on separate annotation rows? Is this useful
#     here at the GPIB level, or shall stacked decoders dispatch these? May
#     depend on how often captures get inspected which involve multiple peers.
#   - Make serial bit annotations optional? Could slow down interactive
#     exploration for long captures (see USB).
# - Move the inlined Commodore IEC peripherals support to a stacked decoder
#   when more peripherals get added.
# - SCPI over GPIB may "represent somewhat naturally" here already when
#   text lines are a single run of data at the GPIB layer (each line between
#   the address spec and either EOI or ATN). So a stacked SCPI decoder may
#   only become necessary when the text lines' content shall get inspected.

import sigrokdecode as srd
from common.srdhelper import bitpack

'''
OUTPUT_PYTHON format for stacked decoders:

General packet format:
[<ptype>, <addr>, <pdata>]

This is the list of <ptype>s and their respective <pdata> values:

Raw bits and bytes at the physical transport level:
 - 'IEC_BIT': <addr> is not applicable, <pdata> is the transport's bit value.
 - 'GPIB_RAW': <addr> is not applicable, <pdata> is the transport's
   byte value. Data bytes are in the 0x00-0xff range, command/address
   bytes are in the 0x100-0x1ff range.

GPIB level byte fields (commands, addresses, pieces of data):
 - 'COMMAND': <addr> is not applicable, <pdata> is the command's byte value.
 - 'LISTEN': <addr> is the listener address (0-30), <pdata> is the raw
   byte value (including the 0x20 offset).
 - 'TALK': <addr> is the talker address (0-30), <pdata> is the raw byte
   value (including the 0x40 offset).
 - 'SECONDARY': <addr> is the secondary address (0-31), <pdata> is the
   raw byte value (including the 0x60 offset).
 - 'MSB_SET': <addr> as well as <pdata> are the raw byte value (including
   the 0x80 offset). This usually does not happen for GPIB bytes with ATN
   active, but was observed with the IEC bus and Commodore floppy drives,
   when addressing channels within the device.
 - 'DATA_BYTE': <addr> is the talker address (when available), <pdata>
   is the raw data byte (transport layer, ATN inactive).

Extracted payload information (peers and their communicated data):
 - 'TALK_LISTEN': <addr> is the current talker, <pdata> is the list of
   current listeners. These updates for the current "connected peers"
   are sent when the set of peers changes, i.e. after talkers/listeners
   got selected or deselected. Of course the data only covers what could
   be gathered from the input data. Some controllers may not explicitly
   address themselves, or captures may not include an early setup phase.
 - 'TALKER_BYTES': <addr> is the talker address (when available), <pdata>
   is the accumulated byte sequence between addressing a talker and EOI,
   or the next command/address.
 - 'TALKER_TEXT': <addr> is the talker address (when available), <pdata>
   is the accumulated text sequence between addressing a talker and EOI,
   or the next command/address.
'''

class ChannelError(Exception):
    pass

def _format_ann_texts(fmts, **args):
    if not fmts:
        return None
    return [fmt.format(**args) for fmt in fmts]

_cmd_table = {
    # Command codes in the 0x00-0x1f range.
    0x01: ['Go To Local', 'GTL'],
    0x04: ['Selected Device Clear', 'SDC'],
    0x05: ['Parallel Poll Configure', 'PPC'],
    0x08: ['Global Execute Trigger', 'GET'],
    0x09: ['Take Control', 'TCT'],
    0x11: ['Local Lock Out', 'LLO'],
    0x14: ['Device Clear', 'DCL'],
    0x15: ['Parallel Poll Unconfigure', 'PPU'],
    0x18: ['Serial Poll Enable', 'SPE'],
    0x19: ['Serial Poll Disable', 'SPD'],
    # Unknown type of command.
    None: ['Unknown command 0x{cmd:02x}', 'command 0x{cmd:02x}', 'cmd {cmd:02x}', 'C{cmd_ord:c}'],
    # Special listener/talker "addresses" (deselecting previous peers).
    0x3f: ['Unlisten', 'UNL'],
    0x5f: ['Untalk', 'UNT'],
}

def _is_command(b):
    # Returns a tuple of booleans (or None when not applicable) whether
    # the raw GPIB byte is: a command, an un-listen, an un-talk command.
    if b in range(0x00, 0x20):
        return True, None, None
    if b in range(0x20, 0x40) and (b & 0x1f) == 31:
        return True, True, False
    if b in range(0x40, 0x60) and (b & 0x1f) == 31:
        return True, False, True
    return False, None, None

def _is_listen_addr(b):
    if b in range(0x20, 0x40):
        return b & 0x1f
    return None

def _is_talk_addr(b):
    if b in range(0x40, 0x60):
        return b & 0x1f
    return None

def _is_secondary_addr(b):
    if b in range(0x60, 0x80):
        return b & 0x1f
    return None

def _is_msb_set(b):
    if b & 0x80:
        return b
    return None

def _get_raw_byte(b, atn):
    # "Decorate" raw byte values for stacked decoders.
    return b | 0x100 if atn else b

def _get_raw_text(b, atn):
    return ['{leader}{data:02x}'.format(leader = '/' if atn else '', data = b)]

def _get_command_texts(b):
    fmts = _cmd_table.get(b, None)
    known = fmts is not None
    if not fmts:
        fmts = _cmd_table.get(None, None)
    if not fmts:
        return known, None
    return known, _format_ann_texts(fmts, cmd = b, cmd_ord = ord('0') + b)

def _get_address_texts(b):
    laddr = _is_listen_addr(b)
    taddr = _is_talk_addr(b)
    saddr = _is_secondary_addr(b)
    msb = _is_msb_set(b)
    fmts = None
    if laddr is not None:
        fmts = ['Listen {addr:d}', 'L {addr:d}', 'L{addr_ord:c}']
        addr = laddr
    elif taddr is not None:
        fmts = ['Talk {addr:d}', 'T {addr:d}', 'T{addr_ord:c}']
        addr = taddr
    elif saddr is not None:
        fmts = ['Secondary {addr:d}', 'S {addr:d}', 'S{addr_ord:c}']
        addr = saddr
    elif msb is not None: # For IEC bus compat.
        fmts = ['Secondary {addr:d}', 'S {addr:d}', 'S{addr_ord:c}']
        addr = msb
    return _format_ann_texts(fmts, addr = addr, addr_ord = ord('0') + addr)

def _get_data_text(b):
    # TODO Move the table of ASCII control characters to a common location?
    # TODO Move the "printable with escapes" logic to a common helper?
    _control_codes = {
        0x00: 'NUL',
        0x01: 'SOH',
        0x02: 'STX',
        0x03: 'ETX',
        0x04: 'EOT',
        0x05: 'ENQ',
        0x06: 'ACK',
        0x07: 'BEL',
        0x08: 'BS',
        0x09: 'TAB',
        0x0a: 'LF',
        0x0b: 'VT',
        0x0c: 'FF',
        0x0d: 'CR',
        0x0e: 'SO',
        0x0f: 'SI',
        0x10: 'DLE',
        0x11: 'DC1',
        0x12: 'DC2',
        0x13: 'DC3',
        0x14: 'DC4',
        0x15: 'NAK',
        0x16: 'SYN',
        0x17: 'ETB',
        0x18: 'CAN',
        0x19: 'EM',
        0x1a: 'SUB',
        0x1b: 'ESC',
        0x1c: 'FS',
        0x1d: 'GS',
        0x1e: 'RS',
        0x1f: 'US',
    }
    # Yes, exclude 0x7f (DEL) here. It's considered non-printable.
    if b in range(0x20, 0x7f) and b not in ('[', ']'):
        return '{:s}'.format(chr(b))
    elif b in _control_codes:
        return '[{:s}]'.format(_control_codes[b])
    # Use a compact yet readable and unambigous presentation for bytes
    # which contain non-printables. The format that is used here is
    # compatible with 93xx EEPROM and UART decoders.
    return '[{:02x}]'.format(b)

(
    PIN_DIO1, PIN_DIO2, PIN_DIO3, PIN_DIO4,
    PIN_DIO5, PIN_DIO6, PIN_DIO7, PIN_DIO8,
    PIN_EOI, PIN_DAV, PIN_NRFD, PIN_NDAC,
    PIN_IFC, PIN_SRQ, PIN_ATN, PIN_REN,
    PIN_CLK,
) = range(17)
PIN_DATA = PIN_DIO1

(
    ANN_RAW_BIT, ANN_RAW_BYTE,
    ANN_CMD, ANN_LADDR, ANN_TADDR, ANN_SADDR, ANN_DATA,
    ANN_EOI,
    ANN_TEXT,
    # TODO Want to provide one annotation class per talker address (0-30)?
    ANN_IEC_PERIPH,
    ANN_WARN,
) = range(11)

(
    BIN_RAW,
    BIN_DATA,
    # TODO Want to provide one binary annotation class per talker address (0-30)?
) = range(2)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'ieee488'
    name = 'IEEE-488'
    longname = 'IEEE-488 GPIB/HPIB/IEC'
    desc = 'IEEE-488 General Purpose Interface Bus (GPIB/HPIB or IEC).'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = ['ieee488']
    tags = ['PC', 'Retro computing']
    channels = (
        {'id': 'dio1' , 'name': 'DIO1/DATA',
            'desc': 'Data I/O bit 1, or serial data'},
    )
    optional_channels = (
        {'id': 'dio2' , 'name': 'DIO2', 'desc': 'Data I/O bit 2'},
        {'id': 'dio3' , 'name': 'DIO3', 'desc': 'Data I/O bit 3'},
        {'id': 'dio4' , 'name': 'DIO4', 'desc': 'Data I/O bit 4'},
        {'id': 'dio5' , 'name': 'DIO5', 'desc': 'Data I/O bit 5'},
        {'id': 'dio6' , 'name': 'DIO6', 'desc': 'Data I/O bit 6'},
        {'id': 'dio7' , 'name': 'DIO7', 'desc': 'Data I/O bit 7'},
        {'id': 'dio8' , 'name': 'DIO8', 'desc': 'Data I/O bit 8'},
        {'id': 'eoi', 'name': 'EOI', 'desc': 'End or identify'},
        {'id': 'dav', 'name': 'DAV', 'desc': 'Data valid'},
        {'id': 'nrfd', 'name': 'NRFD', 'desc': 'Not ready for data'},
        {'id': 'ndac', 'name': 'NDAC', 'desc': 'Not data accepted'},
        {'id': 'ifc', 'name': 'IFC', 'desc': 'Interface clear'},
        {'id': 'srq', 'name': 'SRQ', 'desc': 'Service request'},
        {'id': 'atn', 'name': 'ATN', 'desc': 'Attention'},
        {'id': 'ren', 'name': 'REN', 'desc': 'Remote enable'},
        {'id': 'clk', 'name': 'CLK', 'desc': 'Serial clock'},
    )
    options = (
        {'id': 'iec_periph', 'desc': 'Decode Commodore IEC bus peripherals details',
            'default': 'no', 'values': ('no', 'yes')},
        {'id': 'delim', 'desc': 'Payload data delimiter',
            'default': 'eol', 'values': ('none', 'eol')},
    )
    annotations = (
        ('bit', 'IEC bit'),
        ('raw', 'Raw byte'),
        ('cmd', 'Command'),
        ('laddr', 'Listener address'),
        ('taddr', 'Talker address'),
        ('saddr', 'Secondary address'),
        ('data', 'Data byte'),
        ('eoi', 'EOI'),
        ('text', 'Talker text'),
        ('periph', 'IEC bus peripherals'),
        ('warn', 'Warning'),
    )
    annotation_rows = (
        ('bits', 'IEC bits', (ANN_RAW_BIT,)),
        ('raws', 'Raw bytes', (ANN_RAW_BYTE,)),
        ('gpib', 'Commands/data', (ANN_CMD, ANN_LADDR, ANN_TADDR, ANN_SADDR, ANN_DATA,)),
        ('eois', 'EOI', (ANN_EOI,)),
        ('texts', 'Talker texts', (ANN_TEXT,)),
        ('periphs', 'IEC peripherals', (ANN_IEC_PERIPH,)),
        ('warns', 'Warnings', (ANN_WARN,)),
    )
    binary = (
        ('raw', 'Raw bytes'),
        ('data', 'Talker bytes'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.curr_raw = None
        self.curr_atn = None
        self.curr_eoi = None
        self.latch_atn = None
        self.latch_eoi = None
        self.accu_bytes = []
        self.accu_text = []
        self.ss_raw = None
        self.es_raw = None
        self.ss_eoi = None
        self.es_eoi = None
        self.ss_text = None
        self.es_text = None
        self.last_talker = None
        self.last_listener = []
        self.last_iec_addr = None
        self.last_iec_sec = None

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_bin = self.register(srd.OUTPUT_BINARY)
        self.out_python = self.register(srd.OUTPUT_PYTHON)

    def putg(self, ss, es, data):
        self.put(ss, es, self.out_ann, data)

    def putbin(self, ss, es, data):
        self.put(ss, es, self.out_bin, data)

    def putpy(self, ss, es, ptype, addr, pdata):
        self.put(ss, es, self.out_python, [ptype, addr, pdata])

    def emit_eoi_ann(self, ss, es):
        self.putg(ss, es, [ANN_EOI, ['EOI']])

    def emit_bin_ann(self, ss, es, ann_cls, data):
        self.putbin(ss, es, [ann_cls, bytes(data)])

    def emit_data_ann(self, ss, es, ann_cls, data):
        self.putg(ss, es, [ann_cls, data])

    def emit_warn_ann(self, ss, es, data):
        self.putg(ss, es, [ANN_WARN, data])

    def flush_bytes_text_accu(self):
        if self.accu_bytes and self.ss_text is not None and self.es_text is not None:
            self.emit_bin_ann(self.ss_text, self.es_text, BIN_DATA, bytearray(self.accu_bytes))
            self.putpy(self.ss_text, self.es_text, 'TALKER_BYTES', self.last_talker, bytearray(self.accu_bytes))
            self.accu_bytes = []
        if self.accu_text and self.ss_text is not None and self.es_text is not None:
            text = ''.join(self.accu_text)
            self.emit_data_ann(self.ss_text, self.es_text, ANN_TEXT, [text])
            self.putpy(self.ss_text, self.es_text, 'TALKER_TEXT', self.last_talker, text)
            self.accu_text = []
        self.ss_text = self.es_text = None

    def check_extra_flush(self, b):
        # Optionally flush previously accumulated runs of payload data
        # according to user specified conditions.
        if self.options['delim'] == 'none':
            return
        if not self.accu_bytes:
            return

        # This implementation exlusively handles "text lines", but adding
        # support for more variants here is straight forward.
        #
        # Search for the first data byte _after_ a user specified text
        # line termination sequence was seen. The termination sequence's
        # alphabet may be variable, and the sequence may span multiple
        # data bytes. We accept either CR or LF, and combine the CR+LF
        # sequence to strive for maximum length annotations for improved
        # readability at different zoom levels. It's acceptable that this
        # implementation would also combine multiple line terminations
        # like LF+LF.
        term_chars = (10, 13)
        is_eol = b in term_chars
        had_eol = self.accu_bytes[-1] in term_chars
        if had_eol and not is_eol:
            self.flush_bytes_text_accu()

    def handle_ifc_change(self, ifc):
        # Track IFC line for parallel input.
        # Assertion of IFC de-selects all talkers and listeners.
        if ifc:
            self.last_talker = None
            self.last_listener = []
            self.flush_bytes_text_accu()

    def handle_eoi_change(self, eoi):
        # Track EOI line for parallel and serial input.
        if eoi:
            self.ss_eoi = self.samplenum
            self.curr_eoi = eoi
        else:
            self.es_eoi = self.samplenum
            if self.ss_eoi and self.latch_eoi:
               self.emit_eoi_ann(self.ss_eoi, self.es_eoi)
            self.es_text = self.es_eoi
            self.flush_bytes_text_accu()
            self.ss_eoi = self.es_eoi = None
            self.curr_eoi = None

    def handle_atn_change(self, atn):
        # Track ATN line for parallel and serial input.
        self.curr_atn = atn
        if atn:
            self.flush_bytes_text_accu()

    def handle_iec_periph(self, ss, es, addr, sec, data):
        # The annotation is optional.
        if self.options['iec_periph'] != 'yes':
            return
        # Void internal state.
        if addr is None and sec is None and data is None:
            self.last_iec_addr = None
            self.last_iec_sec = None
            return
        # Grab and evaluate new input.
        _iec_addr_names = {
            # TODO Add more items here. See the "Device numbering" section
            # of the https://en.wikipedia.org/wiki/Commodore_bus page.
            8: 'Disk 0',
            9: 'Disk 1',
        }
        _iec_disk_range = range(8, 16)
        if addr is not None:
            self.last_iec_addr = addr
            name = _iec_addr_names.get(addr, None)
            if name:
                self.emit_data_ann(ss, es, ANN_IEC_PERIPH, [name])
        addr = self.last_iec_addr # Simplify subsequent logic.
        if sec is not None:
            # BEWARE! The secondary address is a full byte and includes
            # the 0x60 offset, to also work when the MSB was set.
            self.last_iec_sec = sec
            subcmd, channel = sec & 0xf0, sec & 0x0f
            channel_ord = ord('0') + channel
            if addr is not None and addr in _iec_disk_range:
                subcmd_fmts = {
                    0x60: ['Reopen {ch:d}', 'Re {ch:d}', 'R{ch_ord:c}'],
                    0xe0: ['Close {ch:d}', 'Cl {ch:d}', 'C{ch_ord:c}'],
                    0xf0: ['Open {ch:d}', 'Op {ch:d}', 'O{ch_ord:c}'],
                }.get(subcmd, None)
                if subcmd_fmts:
                    texts = _format_ann_texts(subcmd_fmts, ch = channel, ch_ord = channel_ord)
                    self.emit_data_ann(ss, es, ANN_IEC_PERIPH, texts)
        sec = self.last_iec_sec # Simplify subsequent logic.
        if data is not None:
            if addr is None or sec is None:
                return
            # TODO Process data depending on peripheral type and channel?

    def handle_data_byte(self):
        if not self.curr_atn:
            self.check_extra_flush(self.curr_raw)
        b = self.curr_raw
        texts = _get_raw_text(b, self.curr_atn)
        self.emit_data_ann(self.ss_raw, self.es_raw, ANN_RAW_BYTE, texts)
        self.emit_bin_ann(self.ss_raw, self.es_raw, BIN_RAW, b.to_bytes(1, byteorder='big'))
        self.putpy(self.ss_raw, self.es_raw, 'GPIB_RAW', None, _get_raw_byte(b, self.curr_atn))
        if self.curr_atn:
            ann_cls = None
            upd_iec = False,
            py_type = None
            py_peers = False
            is_cmd, is_unl, is_unt = _is_command(b)
            laddr = _is_listen_addr(b)
            taddr = _is_talk_addr(b)
            saddr = _is_secondary_addr(b)
            msb = _is_msb_set(b)
            if is_cmd:
                known, texts = _get_command_texts(b)
                if not known:
                    warn_texts = ['Unknown GPIB command', 'unknown', 'UNK']
                    self.emit_warn_ann(self.ss_raw, self.es_raw, warn_texts)
                ann_cls = ANN_CMD
                py_type, py_addr = 'COMMAND', None
                if is_unl:
                    self.last_listener = []
                    py_peers = True
                if is_unt:
                    self.last_talker = None
                    py_peers = True
                if is_unl or is_unt:
                    upd_iec = True, None, None, None
            elif laddr is not None:
                addr = laddr
                texts = _get_address_texts(b)
                ann_cls = ANN_LADDR
                py_type, py_addr = 'LISTEN', addr
                if addr == self.last_talker:
                    self.last_talker = None
                self.last_listener.append(addr)
                upd_iec = True, addr, None, None
                py_peers = True
            elif taddr is not None:
                addr = taddr
                texts = _get_address_texts(b)
                ann_cls = ANN_TADDR
                py_type, py_addr = 'TALK', addr
                if addr in self.last_listener:
                    self.last_listener.remove(addr)
                self.last_talker = addr
                upd_iec = True, addr, None, None
                py_peers = True
            elif saddr is not None:
                addr = saddr
                texts = _get_address_texts(b)
                ann_cls = ANN_SADDR
                upd_iec = True, None, b, None
                py_type, py_addr = 'SECONDARY', addr
            elif msb is not None:
                # These are not really "secondary addresses", but they
                # are used by the Commodore IEC bus (floppy channels).
                texts = _get_address_texts(b)
                ann_cls = ANN_SADDR
                upd_iec = True, None, b, None
                py_type, py_addr = 'MSB_SET', b
            if ann_cls is not None and texts is not None:
                self.emit_data_ann(self.ss_raw, self.es_raw, ann_cls, texts)
            if upd_iec[0]:
                self.handle_iec_periph(self.ss_raw, self.es_raw, upd_iec[1], upd_iec[2], upd_iec[3])
            if py_type:
                self.putpy(self.ss_raw, self.es_raw, py_type, py_addr, b)
            if py_peers:
                self.last_listener.sort()
                self.putpy(self.ss_raw, self.es_raw, 'TALK_LISTEN', self.last_talker, self.last_listener)
        else:
            self.accu_bytes.append(b)
            text = _get_data_text(b)
            if not self.accu_text:
                self.ss_text = self.ss_raw
            self.accu_text.append(text)
            self.es_text = self.es_raw
            self.emit_data_ann(self.ss_raw, self.es_raw, ANN_DATA, [text])
            self.handle_iec_periph(self.ss_raw, self.es_raw, None, None, b)
            self.putpy(self.ss_raw, self.es_raw, 'DATA_BYTE', self.last_talker, b)

    def handle_dav_change(self, dav, data):
        if dav:
            # Data availability starts when the flag goes active.
            self.ss_raw = self.samplenum
            self.curr_raw = bitpack(data)
            self.latch_atn = self.curr_atn
            self.latch_eoi = self.curr_eoi
            return
        # Data availability ends when the flag goes inactive. Handle the
        # previously captured data byte according to associated flags.
        self.es_raw = self.samplenum
        self.handle_data_byte()
        self.ss_raw = self.es_raw = None
        self.curr_raw = None

    def inject_dav_phase(self, ss, es, data):
        # Inspection of serial input has resulted in one raw byte which
        # spans a given period of time. Pretend we had seen a DAV active
        # phase, to re-use code for the parallel transmission.
        self.ss_raw = ss
        self.curr_raw = bitpack(data)
        self.latch_atn = self.curr_atn
        self.latch_eoi = self.curr_eoi
        self.es_raw = es
        self.handle_data_byte()
        self.ss_raw = self.es_raw = None
        self.curr_raw = None

    def invert_pins(self, pins):
        # All lines (including data bits!) are low active and thus need
        # to get inverted to receive their logical state (high active,
        # regular data bit values). Cope with inputs being optional.
        return [1 - p if p in (0, 1) else p for p in pins]

    def decode_serial(self, has_clk, has_data_1, has_atn, has_srq):
        if not has_clk or not has_data_1 or not has_atn:
            raise ChannelError('IEC bus needs at least ATN and serial CLK and DATA.')

        # This is a rephrased version of decoders/iec/pd.py:decode().
        # SRQ was not used there either. Magic numbers were eliminated.
        (
            STEP_WAIT_READY_TO_SEND,
            STEP_WAIT_READY_FOR_DATA,
            STEP_PREP_DATA_TEST_EOI,
            STEP_CLOCK_DATA_BITS,
        ) = range(4)
        step_wait_conds = (
            [{PIN_ATN: 'f'}, {PIN_DATA: 'l', PIN_CLK: 'h'}],
            [{PIN_ATN: 'f'}, {PIN_DATA: 'h', PIN_CLK: 'h'}, {PIN_CLK: 'l'}],
            [{PIN_ATN: 'f'}, {PIN_DATA: 'f'}, {PIN_CLK: 'l'}],
            [{PIN_ATN: 'f'}, {PIN_CLK: 'e'}],
        )
        step = STEP_WAIT_READY_TO_SEND
        bits = []

        while True:

            # Sample input pin values. Keep DATA/CLK in verbatim form to
            # re-use 'iec' decoder logic. Turn ATN to positive logic for
            # easier processing. The data bits get handled during byte
            # accumulation.
            pins = self.wait(step_wait_conds[step])
            data, clk = pins[PIN_DATA], pins[PIN_CLK]
            atn, = self.invert_pins([pins[PIN_ATN]])

            if self.matched[0]:
                # Falling edge on ATN, reset step.
                step = STEP_WAIT_READY_TO_SEND

            if step == STEP_WAIT_READY_TO_SEND:
                # Don't use self.matched[1] here since we might come from
                # a step with different conds due to the code above.
                if data == 0 and clk == 1:
                    # Rising edge on CLK while DATA is low: Ready to send.
                    step = STEP_WAIT_READY_FOR_DATA
            elif step == STEP_WAIT_READY_FOR_DATA:
                if data == 1 and clk == 1:
                    # Rising edge on DATA while CLK is high: Ready for data.
                    ss_byte = self.samplenum
                    self.handle_atn_change(atn)
                    if self.curr_eoi:
                        self.handle_eoi_change(False)
                    bits = []
                    step = STEP_PREP_DATA_TEST_EOI
                elif clk == 0:
                    # CLK low again, transfer aborted.
                    step = STEP_WAIT_READY_TO_SEND
            elif step == STEP_PREP_DATA_TEST_EOI:
                if data == 0 and clk == 1:
                    # DATA goes low while CLK is still high, EOI confirmed.
                    self.handle_eoi_change(True)
                elif clk == 0:
                    step = STEP_CLOCK_DATA_BITS
                    ss_bit = self.samplenum
            elif step == STEP_CLOCK_DATA_BITS:
                if self.matched[1]:
                    if clk == 1:
                        # Rising edge on CLK; latch DATA.
                        bits.append(data)
                    elif clk == 0:
                        # Falling edge on CLK; end of bit.
                        es_bit = self.samplenum
                        self.emit_data_ann(ss_bit, es_bit, ANN_RAW_BIT, ['{:d}'.format(bits[-1])])
                        self.putpy(ss_bit, es_bit, 'IEC_BIT', None, bits[-1])
                        ss_bit = self.samplenum
                        if len(bits) == 8:
                            es_byte = self.samplenum
                            self.inject_dav_phase(ss_byte, es_byte, bits)
                            if self.curr_eoi:
                                self.handle_eoi_change(False)
                            step = STEP_WAIT_READY_TO_SEND

    def decode_parallel(self, has_data_n, has_dav, has_atn, has_eoi, has_srq):

        if False in has_data_n or not has_dav or not has_atn:
            raise ChannelError('IEEE-488 needs at least ATN and DAV and eight DIO lines.')
        has_ifc = self.has_channel(PIN_IFC)

        # Capture data lines at the falling edge of DAV, process their
        # values at rising DAV edge (when data validity ends). Also make
        # sure to start inspection when the capture happens to start with
        # low signal levels, i.e. won't include the initial falling edge.
        # Scan for ATN/EOI edges as well (including the trick which works
        # around initial pin state).
        # Map low-active physical transport lines to positive logic here,
        # to simplify logical inspection/decoding of communicated data,
        # and to avoid redundancy and inconsistency in later code paths.
        waitcond = []
        idx_dav = len(waitcond)
        waitcond.append({PIN_DAV: 'l'})
        idx_atn = len(waitcond)
        waitcond.append({PIN_ATN: 'l'})
        idx_eoi = None
        if has_eoi:
            idx_eoi = len(waitcond)
            waitcond.append({PIN_EOI: 'l'})
        idx_ifc = None
        if has_ifc:
            idx_ifc = len(waitcond)
            waitcond.append({PIN_IFC: 'l'})
        while True:
            pins = self.wait(waitcond)
            pins = self.invert_pins(pins)

            # BEWARE! Order of evaluation does matter. For low samplerate
            # captures, many edges fall onto the same sample number. So
            # we process active edges of flags early (before processing
            # data bits), and inactive edges late (after data got processed).
            if idx_ifc is not None and self.matched[idx_ifc] and pins[PIN_IFC] == 1:
                self.handle_ifc_change(pins[PIN_IFC])
            if idx_eoi is not None and self.matched[idx_eoi] and pins[PIN_EOI] == 1:
                self.handle_eoi_change(pins[PIN_EOI])
            if self.matched[idx_atn] and pins[PIN_ATN] == 1:
                self.handle_atn_change(pins[PIN_ATN])
            if self.matched[idx_dav]:
                self.handle_dav_change(pins[PIN_DAV], pins[PIN_DIO1:PIN_DIO8 + 1])
            if self.matched[idx_atn] and pins[PIN_ATN] == 0:
                self.handle_atn_change(pins[PIN_ATN])
            if idx_eoi is not None and self.matched[idx_eoi] and pins[PIN_EOI] == 0:
                self.handle_eoi_change(pins[PIN_EOI])
            if idx_ifc is not None and self.matched[idx_ifc] and pins[PIN_IFC] == 0:
                self.handle_ifc_change(pins[PIN_IFC])

            waitcond[idx_dav][PIN_DAV] = 'e'
            waitcond[idx_atn][PIN_ATN] = 'e'
            if has_eoi:
                waitcond[idx_eoi][PIN_EOI] = 'e'
            if has_ifc:
                waitcond[idx_ifc][PIN_IFC] = 'e'

    def decode(self):
        # The decoder's boilerplate declares some of the input signals as
        # optional, but only to support both serial and parallel variants.
        # The CLK signal discriminates the two. For either variant some
        # of the "optional" signals are not really optional for proper
        # operation of the decoder. Check these conditions here.
        has_clk = self.has_channel(PIN_CLK)
        has_data_1 = self.has_channel(PIN_DIO1)
        has_data_n = [bool(self.has_channel(pin) for pin in range(PIN_DIO1, PIN_DIO8 + 1))]
        has_dav = self.has_channel(PIN_DAV)
        has_atn = self.has_channel(PIN_ATN)
        has_eoi = self.has_channel(PIN_EOI)
        has_srq = self.has_channel(PIN_SRQ)
        if has_clk:
            self.decode_serial(has_clk, has_data_1, has_atn, has_srq)
        else:
            self.decode_parallel(has_data_n, has_dav, has_atn, has_eoi, has_srq)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2013 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
This protocol decoder tries to guess the bitrate / baudrate of the
communication on the specified channel.

Typically this will be used to guess / detect the baudrate used in a UART
communication snippet, but it could also be used to guess bitrates of certain
other protocols or buses.

It should be noted that this is nothing more than a simple guess / heuristic,
and that there are various cases in practice where the detection of the
bitrate or baudrate will not necessarily have the expected result.

The precision of the estimated bitrate / baudrate will also depend on the
samplerate used to sample the respective channel. For good results it is
recommended to use a logic analyzer samplerate that is much higher than
the expected bitrate/baudrate that might be used on the channel.

The last annotation emitted by the decoder will be the best bitrate guess.
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from common.sdcard import (cmd_names, acmd_names, accepted_voltages, card_status, sd_status)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'sdcard_sd'
    name = 'SD card (SD mode)'
    longname = 'Secure Digital card (SD mode)'
    desc = 'Secure Digital card (SD mode) low-level protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Memory']
    channels = (
        {'id': 'cmd',  'name': 'CMD',  'desc': 'Command'},
        {'id': 'clk',  'name': 'CLK',  'desc': 'Clock'},
    )
    optional_channels = (
        {'id': 'dat0', 'name': 'DAT0', 'desc': 'Data pin 0'},
        {'id': 'dat1', 'name': 'DAT1', 'desc': 'Data pin 1'},
        {'id': 'dat2', 'name': 'DAT2', 'desc': 'Data pin 2'},
        {'id': 'dat3', 'name': 'DAT3', 'desc': 'Data pin 3'},
    )
    annotations = \
        tuple(('cmd%d' % i, 'CMD%d' % i) for i in range(64)) + \
        tuple(('acmd%d' % i, 'ACMD%d' % i) for i in range(64)) + ( \
        ('bits', 'Bits'),
        ('field-start', 'Start bit'),
        ('field-transmission', 'Transmission bit'),
        ('field-cmd', 'Command'),
        ('field-arg', 'Argument'),
        ('field-crc', 'CRC'),
        ('field-end', 'End bit'),
        ('decoded-bits', 'Decoded bits'),
        ('decoded-fields', 'Decoded fields'),
    )
    annotation_rows = (
        ('raw-bits', 'Raw bits', (128,)),
        ('decoded-bits', 'Decoded bits', (135,)),
        ('decoded-fields', 'Decoded fields', (136,)),
        ('fields', 'Fields', tuple(range(129, 135))),
        ('cmd', 'Commands', tuple(range(128))),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'GET COMMAND TOKEN'
        self.token = []
        self.is_acmd = False # Indicates CMD vs. ACMD
        self.cmd = None
        self.last_cmd = None
        self.arg = None

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putbit(self, b, data):
        self.put(self.token[b][0], self.token[b][1], self.out_ann, [135, data])

    def putt(self, data):
        self.put(self.token[0][0], self.token[47][1], self.out_ann, data)

    def putf(self, s, e, data):
        self.put(self.token[s][0], self.token[e][1], self.out_ann, data)

    def puta(self, s, e, data):
        self.put(self.token[47 - 8 - e][0], self.token[47 - 8 - s][1],
                 self.out_ann, data)

    def putc(self, cmd, desc):
        self.last_cmd = cmd
        self.putt([cmd, ['%s: %s' % (self.cmd_str, desc), self.cmd_str,
                         self.cmd_str.split(' ')[0]]])

    def putr(self, desc):
        self.putt([self.last_cmd, ['Reply: %s' % desc]])

    def cmd_name(self, cmd):
        c = acmd_names if self.is_acmd else cmd_names
        return c.get(cmd, 'Unknown')

    def get_token_bits(self, cmd, n):
        # Get a bit, return True if we already got 'n' bits, False otherwise.
        self.token.append([self.samplenum, self.samplenum, cmd])
        if len(self.token) > 0:
            self.token[len(self.token) - 2][1] = self.samplenum
        if len(self.token) < n:
            return False
        self.token[n - 1][1] += self.token[n - 1][0] - self.token[n - 2][0]
        return True

    def handle_common_token_fields(self):
        s = self.token

        # Annotations for each individual bit.
        for bit in range(len(self.token)):
            self.putf(bit, bit, [128, ['%d' % s[bit][2]]])

        # CMD[47:47]: Start bit (always 0)
        self.putf(0, 0, [129, ['Start bit', 'Start', 'S']])

        # CMD[46:46]: Transmission bit (1 == host)
        t = 'host' if s[1][2] == 1 else 'card'
        self.putf(1, 1, [130, ['Transmission: ' + t, 'T: ' + t, 'T']])

        # CMD[45:40]: Command index (BCD; valid: 0-63)
        self.cmd = int('0b' + ''.join([str(s[i][2]) for i in range(2, 8)]), 2)
        c = '%s (%d)' % (self.cmd_name(self.cmd), self.cmd)
        self.putf(2, 7, [131, ['Command: ' + c, 'Cmd: ' + c,
                               'CMD%d' % self.cmd, 'Cmd', 'C']])

        # CMD[39:08]: Argument
        self.arg = int('0b' + ''.join([str(s[i][2]) for i in range(8, 40)]), 2)
        self.putf(8, 39, [132, ['Argument: 0x%08x' % self.arg, 'Arg', 'A']])

        # CMD[07:01]: CRC7
        self.crc = int('0b' + ''.join([str(s[i][2]) for i in range(40, 47)]), 2)
        self.putf(40, 46, [133, ['CRC: 0x%x' % self.crc, 'CRC', 'C']])

        # CMD[00:00]: End bit (always 1)
        self.putf(47, 47, [134, ['End bit', 'End', 'E']])

    def get_command_token(self, cmd):
        # Command tokens (48 bits) are sent serially (MSB-first) by the host
        # (over the CMD line), either to one SD card or to multiple ones.
        #
        # Format:
        #  - Bits[47:47]: Start bit (always 0)
        #  - Bits[46:46]: Transmission bit (1 == host)
        #  - Bits[45:40]: Command index (BCD; valid: 0-63)
        #  - Bits[39:08]: Argument
        #  - Bits[07:01]: CRC7
        #  - Bits[00:00]: End bit (always 1)

        if not self.get_token_bits(cmd, 48):
            return

        self.handle_common_token_fields()

        # Handle command.
        s = 'ACMD' if self.is_acmd else 'CMD'
        self.cmd_str = '%s%d (%s)' % (s, self.cmd, self.cmd_name(self.cmd))
        if self.cmd in (0, 2, 3, 6, 7, 8, 9, 10, 13, 41, 51, 55):
            self.state = 'HANDLE CMD%d' % self.cmd
        else:
            self.state = 'HANDLE CMD999'
            self.putc(self.cmd, '%s%d' % (s, self.cmd))

    def handle_cmd0(self):
        # CMD0 (GO_IDLE_STATE) -> no response
        self.puta(0, 31, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(0, 'Reset all SD cards')
        self.token, self.state = [], 'GET COMMAND TOKEN'

    def handle_cmd2(self):
        # CMD2 (ALL_SEND_CID) -> R2
        self.puta(0, 31, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(2, 'Ask card for CID number')
        self.token, self.state = [], 'GET RESPONSE R2'

    def handle_cmd3(self):
        # CMD3 (SEND_RELATIVE_ADDR) -> R6
        self.puta(0, 31, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(3, 'Ask card for new relative card address (RCA)')
        self.token, self.state = [], 'GET RESPONSE R6'

    def handle_cmd6(self):
        # CMD6 (SWITCH_FUNC) -> R1
        self.putc(6, 'Switch/check card function')
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_cmd7(self):
        # CMD7 (SELECT/DESELECT_CARD) -> R1b
        self.putc(7, 'Select / deselect card')
        self.token, self.state = [], 'GET RESPONSE R6'

    def handle_cmd8(self):
        # CMD8 (SEND_IF_COND) -> R7
        self.puta(12, 31, [136, ['Reserved', 'Res', 'R']])
        self.puta(8, 11, [136, ['Supply voltage', 'Voltage', 'VHS', 'V']])
        self.puta(0, 7, [136, ['Check pattern', 'Check pat', 'Check', 'C']])
        self.putc(8, 'Send interface condition to card')
        self.token, self.state = [], 'GET RESPONSE R7'
        # TODO: Handle case when card doesn't reply with R7 (no reply at all).

    def handle_cmd9(self):
        # CMD9 (SEND_CSD) -> R2
        self.puta(16, 31, [136, ['RCA', 'R']])
        self.puta(0, 15, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(9, 'Send card-specific data (CSD)')
        self.token, self.state = [], 'GET RESPONSE R2'

    def handle_cmd10(self):
        # CMD10 (SEND_CID) -> R2
        self.puta(16, 31, [136, ['RCA', 'R']])
        self.puta(0, 15, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(9, 'Send card identification data (CID)')
        self.token, self.state = [], 'GET RESPONSE R2'

    def handle_cmd13(self):
        # CMD13 (SEND_STATUS) -> R1
        self.puta(16, 31, [136, ['RCA', 'R']])
        self.puta(0, 15, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(13, 'Send card status register')
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_cmd16(self):
        # CMD16 (SET_BLOCKLEN) -> R1
        self.puta(0, 31, [136, ['Block length', 'Blocklen', 'BL', 'B']])
        self.putc(16, 'Set the block length to %d bytes' % self.arg)
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_cmd55(self):
        # CMD55 (APP_CMD) -> R1
        self.puta(16, 31, [136, ['RCA', 'R']])
        self.puta(0, 15, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(55, 'Next command is an application-specific command')
        self.is_acmd = True
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_acmd6(self):
        # ACMD6 (SET_BUS_WIDTH) -> R1
        self.putc(64 + 6, 'Read SD config register (SCR)')
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_acmd13(self):
        # ACMD13 (SD_STATUS) -> R1
        self.puta(0, 31, [136, ['Stuff bits', 'Stuff', 'SB', 'S']])
        self.putc(64 + 13, 'Set SD status')
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_acmd41(self):
        # ACMD41 (SD_SEND_OP_COND) -> R3
        self.puta(0, 23, [136, ['VDD voltage window', 'VDD volt', 'VDD', 'V']])
        self.puta(24, 24, [136, ['S18R']])
        self.puta(25, 27, [136, ['Reserved', 'Res', 'R']])
        self.puta(28, 28, [136, ['XPC']])
        self.puta(29, 29, [136, ['Reserved for eSD', 'Reserved', 'Res', 'R']])
        self.puta(30, 30, [136, ['Host capacity support info', 'Host capacity',
                                 'HCS', 'H']])
        self.puta(31, 31, [136, ['Reserved', 'Res', 'R']])
        self.putc(64 + 41, 'Send HCS info and activate the card init process')
        self.token, self.state = [], 'GET RESPONSE R3'

    def handle_acmd51(self):
        # ACMD51 (SEND_SCR) -> R1
        self.putc(64 + 51, 'Read SD config register (SCR)')
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_cmd999(self):
        self.token, self.state = [], 'GET RESPONSE R1'

    def handle_acmd999(self):
        self.token, self.state = [], 'GET RESPONSE R1'

    # Response tokens can have one of four formats (depends on content).
    # They can have a total length of 48 or 136 bits.
    # They're sent serially (MSB-first) by the card that the host
    # addressed previously, or (synchronously) by all connected cards.

    def handle_response_r1(self, cmd):
        # R1: Normal response command
        #  - Bits[47:47]: Start bit (always 0)
        #  - Bits[46:46]: Transmission bit (0 == card)
        #  - Bits[45:40]: Command index (BCD; valid: 0-63)
        #  - Bits[39:08]: Card status
        #  - Bits[07:01]: CRC7
        #  - Bits[00:00]: End bit (always 1)
        if not self.get_token_bits(cmd, 48):
            return
        self.handle_common_token_fields()
        self.putr('R1')
        self.puta(0, 31, [136, ['Card status', 'Status', 'S']])
        for i in range(32):
            self.putbit(8 + i, [card_status[31 - i]])
        self.token, self.state = [], 'GET COMMAND TOKEN'

    def handle_response_r1b(self, cmd):
        # R1b: Same as R1 with an optional busy signal (on the data line)
        if not self.get_token_bits(cmd, 48):
            return
        self.handle_common_token_fields()
        self.puta(0, 31, [136, ['Card status', 'Status', 'S']])
        self.putr('R1b')
        self.token, self.state = [], 'GET COMMAND TOKEN'

    def handle_response_r2(self, cmd):
        # R2: CID/CSD register
        #  - Bits[135:135]: Start bit (always 0)
        #  - Bits[134:134]: Transmission bit (0 == card)
        #  - Bits[133:128]: Reserved (always 0b111111)
        #  - Bits[127:001]: CID or CSD register including internal CRC7
        #  - Bits[000:000]: End bit (always 1)
        if not self.get_token_bits(cmd, 136):
            return
        # Annotations for each individual bit.
        for bit in range(len(self.token)):
            self.putf(bit, bit, [128, ['%d' % self.token[bit][2]]])
        self.putf(0, 0, [129, ['Start bit', 'Start', 'S']])
        t = 'host' if self.token[1][2] == 1 else 'card'
        self.putf(1, 1, [130, ['Transmission: ' + t, 'T: ' + t, 'T']])
        self.putf(2, 7, [131, ['Reserved', 'Res', 'R']])
        self.putf(8, 134, [132, ['Argument', 'Arg', 'A']])
        self.putf(135, 135, [134, ['End bit', 'End', 'E']])
        self.putf(8, 134, [136, ['CID/CSD register', 'CID/CSD', 'C']])
        self.putf(0, 135, [55, ['R2']])
        self.token, self.state = [], 'GET COMMAND TOKEN'

    def handle_response_r3(self, cmd):
        # R3: OCR register
        #  - Bits[47:47]: Start bit (always 0)
        #  - Bits[46:46]: Transmission bit (0 == card)
        #  - Bits[45:40]: Reserved (always 0b111111)
        #  - Bits[39:08]: OCR register
        #  - Bits[07:01]: Reserved (always 0b111111)
        #  - Bits[00:00]: End bit (always 1)
        if not self.get_token_bits(cmd, 48):
            return
        self.putr('R3')
        # Annotations for each individual bit.
        for bit in range(len(self.token)):
            self.putf(bit, bit, [128, ['%d' % self.token[bit][2]]])
        self.putf(0, 0, [129, ['Start bit', 'Start', 'S']])
        t = 'host' if self.token[1][2] == 1 else 'card'
        self.putf(1, 1, [130, ['Transmission: ' + t, 'T: ' + t, 'T']])
        self.putf(2, 7, [131, ['Reserved', 'Res', 'R']])
        self.putf(8, 39, [132, ['Argument', 'Arg', 'A']])
        self.putf(40, 46, [133, ['Reserved', 'Res', 'R']])
        self.putf(47, 47, [134, ['End bit', 'End', 'E']])
        self.puta(0, 31, [136, ['OCR register', 'OCR reg', 'OCR', 'O']])
        self.token, self.state = [], 'GET COMMAND TOKEN'

    def handle_response_r6(self, cmd):
        # R6: Published RCA response
        #  - Bits[47:47]: Start bit (always 0)
        #  - Bits[46:46]: Transmission bit (0 == card)
        #  - Bits[45:40]: Command index (always 0b000011)
        #  - Bits[39:24]: Argument[31:16]: New published RCA of the card
        #  - Bits[23:08]: Argument[15:0]: Card status bits
        #  - Bits[07:01]: CRC7
        #  - Bits[00:00]: End bit (always 1)
        if not self.get_token_bits(cmd, 48):
            return
        self.handle_common_token_fields()
        self.puta(0, 15, [136, ['Card status bits', 'Status', 'S']])
        self.puta(16, 31, [136, ['Relative card address', 'RCA', 'R']])
        self.putr('R6')
        self.token, self.state = [], 'GET COMMAND TOKEN'

    def handle_response_r7(self, cmd):
        # R7: Card interface condition
        #  - Bits[47:47]: Start bit (always 0)
        #  - Bits[46:46]: Transmission bit (0 == card)
        #  - Bits[45:40]: Command index (always 0b001000)
        #  - Bits[39:20]: Reserved bits (all-zero)
        #  - Bits[19:16]: Voltage accepted
        #  - Bits[15:08]: Echo-back of check pattern
        #  - Bits[07:01]: CRC7
        #  - Bits[00:00]: End bit (always 1)
        if not self.get_token_bits(cmd, 48):
            return
        self.handle_common_token_fields()

        self.putr('R7')

        # Arg[31:12]: Reserved bits (all-zero)
        self.puta(12, 31, [136, ['Reserved', 'Res', 'R']])

        # Arg[11:08]: Voltage accepted
        v = ''.join(str(i[2]) for i in self.token[28:32])
        av = accepted_voltages.get(int('0b' + v, 2), 'Unknown')
        self.puta(8, 11, [136, ['Voltage accepted: ' + av, 'Voltage', 'Volt', 'V']])

        # Arg[07:00]: Echo-back of check pattern
        self.puta(0, 7, [136, ['Echo-back of check pattern', 'Echo', 'E']])

        self.token, self.state = [], 'GET COMMAND TOKEN'

    def decode(self):
        while True:
            # Wait for a rising CLK edge.
            (cmd, clk, dat0, dat1, dat2, dat3) = self.wait({1: 'r'})

            # State machine.
            if self.state == 'GET COMMAND TOKEN':
                if len(self.token) == 0:
                    # Wait for start bit (CMD = 0).
                    if cmd != 0:
                        continue
                self.get_command_token(cmd)
            elif self.state.startswith('HANDLE CMD'):
                # Call the respective handler method for the command.
                a, cmdstr = 'a' if self.is_acmd else '', self.state[10:].lower()
                handle_cmd = getattr(self, 'handle_%scmd%s' % (a, cmdstr))
                handle_cmd()
                # Leave ACMD mode again after the first command after CMD55.
                if self.is_acmd and cmdstr not in ('55', '63'):
                    self.is_acmd = False
            elif self.state.startswith('GET RESPONSE'):
                if len(self.token) == 0:
                    # Wait for start bit (CMD = 0).
                    if cmd != 0:
                        continue
                # Call the respective handler method for the response.
                s = 'handle_response_%s' % self.state[13:].lower()
                handle_response = getattr(self, s)
                handle_response(cmd)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2011-2016 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from .lists import *

L = len(cmds)

# Don't forget to keep this in sync with 'cmds' in lists.py.
class Ann:
    WRSR, PP, READ, WRDI, RDSR, WREN, FAST_READ, SE, RDSCUR, WRSCUR, \
    RDSR2, CE, ESRY, DSRY, WRITE1, WRITE2, REMS, RDID, RDP_RES, CP, ENSO, DP, \
    READ2X, EXSO, CE2, STATUS, BE, REMS2, \
    BIT, FIELD, WARN = range(L + 3)

def cmd_annotation_classes():
    return tuple([tuple([cmd[0].lower(), cmd[1]]) for cmd in cmds.values()])

def decode_dual_bytes(sio0, sio1):
    # Given a byte in SIO0 (MOSI) of even bits and a byte in
    # SIO1 (MISO) of odd bits, return a tuple of two bytes.
    def combine_byte(even, odd):
        result = 0
        for bit in range(4):
            if even & (1 << bit):
                result |= 1 << (bit*2)
            if odd & (1 << bit):
                result |= 1 << ((bit*2) + 1)
        return result
    return (combine_byte(sio0 >> 4, sio1 >> 4), combine_byte(sio0, sio1))

def decode_status_reg(data):
    # TODO: Additional per-bit(s) self.put() calls with correct start/end.

    # Bits[0:0]: WIP (write in progress)
    s = 'W' if (data & (1 << 0)) else 'No w'
    ret = '%srite operation in progress.\n' % s

    # Bits[1:1]: WEL (write enable latch)
    s = '' if (data & (1 << 1)) else 'not '
    ret += 'Internal write enable latch is %sset.\n' % s

    # Bits[5:2]: Block protect bits
    # TODO: More detailed decoding (chip-dependent).
    ret += 'Block protection bits (BP3-BP0): 0x%x.\n' % ((data & 0x3c) >> 2)

    # Bits[6:6]: Continuously program mode (CP mode)
    s = '' if (data & (1 << 6)) else 'not '
    ret += 'Device is %sin continuously program mode (CP mode).\n' % s

    # Bits[7:7]: SRWD (status register write disable)
    s = 'not ' if (data & (1 << 7)) else ''
    ret += 'Status register writes are %sallowed.\n' % s

    return ret

class Decoder(srd.Decoder):
    api_version = 3
    id = 'spiflash'
    name = 'SPI flash/EEPROM'
    longname = 'SPI flash/EEPROM chips'
    desc = 'xx25 series SPI (NOR) flash/EEPROM chip protocol.'
    license = 'gplv2+'
    inputs = ['spi']
    outputs = []
    tags = ['IC', 'Memory']
    annotations = cmd_annotation_classes() + (
        ('bit', 'Bit'),
        ('field', 'Field'),
        ('warning', 'Warning'),
    )
    annotation_rows = (
        ('bits', 'Bits', (L + 0,)),
        ('fields', 'Fields', (L + 1,)),
        ('commands', 'Commands', tuple(range(len(cmds)))),
        ('warnings', 'Warnings', (L + 2,)),
    )
    options = (
        {'id': 'chip', 'desc': 'Chip', 'default': tuple(chips.keys())[0],
            'values': tuple(chips.keys())},
        {'id': 'format', 'desc': 'Data format', 'default': 'hex',
            'values': ('hex', 'ascii')},
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.device_id = -1
        self.on_end_transaction = None
        self.end_current_transaction()
        self.writestate = 0

        # Build dict mapping command keys to handler functions. Each
        # command in 'cmds' (defined in lists.py) has a matching
        # handler self.handle_<shortname>.
        def get_handler(cmd):
            s = 'handle_%s' % cmds[cmd][0].lower().replace('/', '_')
            return getattr(self, s)
        self.cmd_handlers = dict((cmd, get_handler(cmd)) for cmd in cmds.keys())

    def end_current_transaction(self):
        if self.on_end_transaction is not None: # Callback for CS# transition.
            self.on_end_transaction()
            self.on_end_transaction = None
        self.state = None
        self.cmdstate = 1
        self.addr = 0
        self.data = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.chip = chips[self.options['chip']]
        self.vendor = self.options['chip'].split('_')[0]

    def putx(self, data):
        # Simplification, most annotations span exactly one SPI byte/packet.
        self.put(self.ss, self.es, self.out_ann, data)

    def putf(self, data):
        self.put(self.ss_field, self.es_field, self.out_ann, data)

    def putc(self, data):
        self.put(self.ss_cmd, self.es_cmd, self.out_ann, data)

    def device(self):
        return device_name[self.vendor].get(self.device_id, 'Unknown')

    def vendor_device(self):
        return '%s %s' % (self.chip['vendor'], self.device())

    def cmd_ann_list(self):
        x, s = cmds[self.state][0], cmds[self.state][1]
        return ['Command: %s (%s)' % (s, x), 'Command: %s' % s,
                'Cmd: %s' % s, 'Cmd: %s' % x, x]

    def cmd_vendor_dev_list(self):
        c, d = cmds[self.state], 'Device = %s' % self.vendor_device()
        return ['%s (%s): %s' % (c[1], c[0], d), '%s: %s' % (c[1], d),
                '%s: %s' % (c[0], d), d, self.vendor_device()]

    def emit_cmd_byte(self):
        self.ss_cmd = self.ss
        self.putx([Ann.FIELD, self.cmd_ann_list()])
        self.addr = 0

    def emit_addr_bytes(self, mosi):
        self.addr |= (mosi << ((4 - self.cmdstate) * 8))
        b = ((3 - (self.cmdstate - 2)) * 8) - 1
        self.putx([Ann.BIT,
            ['Address bits %d..%d: 0x%02x' % (b, b - 7, mosi),
             'Addr bits %d..%d: 0x%02x' % (b, b - 7, mosi),
             'Addr bits %d..%d' % (b, b - 7), 'A%d..A%d' % (b, b - 7)]])
        if self.cmdstate == 2:
            self.ss_field = self.ss
        if self.cmdstate == 4:
            self.es_field = self.es
            self.putf([Ann.FIELD, ['Address: 0x%06x' % self.addr,
                'Addr: 0x%06x' % self.addr, '0x%06x' % self.addr]])

    def handle_wren(self, mosi, miso):
        self.putx([Ann.WREN, self.cmd_ann_list()])
        self.writestate = 1

    def handle_wrdi(self, mosi, miso):
        self.putx([Ann.WRDI, self.cmd_ann_list()])
        self.writestate = 0

    def handle_rdid(self, mosi, miso):
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate == 2:
            # Byte 2: Slave sends the JEDEC manufacturer ID.
            self.putx([Ann.FIELD, ['Manufacturer ID: 0x%02x' % miso]])
        elif self.cmdstate == 3:
            # Byte 3: Slave sends the memory type.
            self.putx([Ann.FIELD, ['Memory type: 0x%02x' % miso]])
        elif self.cmdstate == 4:
            # Byte 4: Slave sends the device ID.
            self.device_id = miso
            self.putx([Ann.FIELD, ['Device ID: 0x%02x' % miso]])

        if self.cmdstate == 4:
            self.es_cmd = self.es
            self.putc([Ann.RDID, self.cmd_vendor_dev_list()])
            self.state = None
        else:
            self.cmdstate += 1

    def handle_rdsr(self, mosi, miso):
        # Read status register: Master asserts CS#, sends RDSR command,
        # reads status register byte. If CS# is kept asserted, the status
        # register can be read continuously / multiple times in a row.
        # When done, the master de-asserts CS# again.
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate >= 2:
            # Bytes 2-x: Slave sends status register as long as master clocks.
            self.es_cmd = self.es
            self.putx([Ann.BIT, [decode_status_reg(miso)]])
            self.putx([Ann.FIELD, ['Status register']])
            self.putc([Ann.RDSR, self.cmd_ann_list()])
            # Set write latch state.
            self.writestate = 1 if (miso & (1 << 1)) else 0
        self.cmdstate += 1

    def handle_rdsr2(self, mosi, miso):
        # Read status register 2: Master asserts CS#, sends RDSR2 command,
        # reads status register 2 byte. If CS# is kept asserted, the status
        # register 2 can be read continuously / multiple times in a row.
        # When done, the master de-asserts CS# again.
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate >= 2:
            # Bytes 2-x: Slave sends status register 2 as long as master clocks.
            self.es_cmd = self.es
            # TODO: Decode status register 2 correctly.
            self.putx([Ann.BIT, [decode_status_reg(miso)]])
            self.putx([Ann.FIELD, ['Status register 2']])
            self.putc([Ann.RDSR2, self.cmd_ann_list()])
        self.cmdstate += 1

    def handle_wrsr(self, mosi, miso):
        # Write status register: Master asserts CS#, sends WRSR command,
        # writes 1 or 2 status register byte(s).
        # When done, the master de-asserts CS# again. If this doesn't happen
        # the WRSR command will not be executed.
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate == 2:
            # Byte 2: Master sends status register 1.
            self.putx([Ann.BIT, [decode_status_reg(mosi)]])
            self.putx([Ann.FIELD, ['Status register 1']])
            # Set write latch state.
            self.writestate = 1 if (miso & (1 << 1)) else 0
        elif self.cmdstate == 3:
            # Byte 3: Master sends status register 2.
            # TODO: Decode status register 2 correctly.
            self.putx([Ann.BIT, [decode_status_reg(mosi)]])
            self.putx([Ann.FIELD, ['Status register 2']])
            self.es_cmd = self.es
            self.putc([Ann.WRSR, self.cmd_ann_list()])
        self.cmdstate += 1

    def handle_read(self, mosi, miso):
        # Read data bytes: Master asserts CS#, sends READ command, sends
        # 3-byte address, reads >= 1 data bytes, de-asserts CS#.
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate in (2, 3, 4):
            # Bytes 2/3/4: Master sends read address (24bits, MSB-first).
            self.emit_addr_bytes(mosi)
        elif self.cmdstate >= 5:
            # Bytes 5-x: Master reads data bytes (until CS# de-asserted).
            self.es_field = self.es # Will be overwritten for each byte.
            if self.cmdstate == 5:
                self.ss_field = self.ss
                self.on_end_transaction = lambda: self.output_data_block('Data', Ann.READ)
            self.data.append(miso)
        self.cmdstate += 1

    def handle_write_common(self, mosi, miso, ann):
        # Write data bytes: Master asserts CS#, sends WRITE command, sends
        # 3-byte address, writes >= 1 data bytes, de-asserts CS#.
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
            if self.writestate == 0:
                self.putc([Ann.WARN, ['Warning: WREN might be missing']])
        elif self.cmdstate in (2, 3, 4):
            # Bytes 2/3/4: Master sends write address (24bits, MSB-first).
            self.emit_addr_bytes(mosi)
        elif self.cmdstate >= 5:
            # Bytes 5-x: Master writes data bytes (until CS# de-asserted).
            self.es_field = self.es # Will be overwritten for each byte.
            if self.cmdstate == 5:
                self.ss_field = self.ss
                self.on_end_transaction = lambda: self.output_data_block('Data', ann)
            self.data.append(mosi)
        self.cmdstate += 1

    def handle_write1(self, mosi, miso):
        self.handle_write_common(mosi, miso, Ann.WRITE1)

    def handle_write2(self, mosi, miso):
        self.handle_write_common(mosi, miso, Ann.WRITE2)

    def handle_fast_read(self, mosi, miso):
        # Fast read: Master asserts CS#, sends FAST READ command, sends
        # 3-byte address + 1 dummy byte, reads >= 1 data bytes, de-asserts CS#.
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate in (2, 3, 4):
            # Bytes 2/3/4: Master sends read address (24bits, MSB-first).
            self.emit_addr_bytes(mosi)
        elif self.cmdstate == 5:
            self.putx([Ann.BIT, ['Dummy byte: 0x%02x' % mosi]])
        elif self.cmdstate >= 6:
            # Bytes 6-x: Master reads data bytes (until CS# de-asserted).
            self.es_field = self.es # Will be overwritten for each byte.
            if self.cmdstate == 6:
                self.ss_field = self.ss
                self.on_end_transaction = lambda: self.output_data_block('Data', Ann.FAST_READ)
            self.data.append(miso)
        self.cmdstate += 1

    def handle_2read(self, mosi, miso):
        # 2x I/O read (fast read dual I/O): Master asserts CS#, sends 2READ
        # command, sends 3-byte address + 1 dummy byte, reads >= 1 data bytes,
        # de-asserts CS#. All data after the command is sent via two I/O pins.
        # MOSI = SIO0 = even bits, MISO = SIO1 = odd bits.
        if self.cmdstate != 1:
            b1, b2 = decode_dual_bytes(mosi, miso)
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate == 2:
            # Bytes 2/3(/4): Master sends read address (24bits, MSB-first).
            # Handle bytes 2 and 3 here.
            self.emit_addr_bytes(b1)
            self.cmdstate = 3
            self.emit_addr_bytes(b2)
        elif self.cmdstate == 4:
            # Byte 5: Dummy byte. Also handle byte 4 (address LSB) here.
            self.emit_addr_bytes(b1)
            self.cmdstate = 5
            self.putx([Ann.BIT, ['Dummy byte: 0x%02x' % b2]])
        elif self.cmdstate >= 6:
            # Bytes 6-x: Master reads data bytes (until CS# de-asserted).
            self.es_field = self.es # Will be overwritten for each byte.
            if self.cmdstate == 6:
                self.ss_field = self.ss
                self.on_end_transaction = lambda: self.output_data_block('Data', Ann.READ2X)
            self.data.append(b1)
            self.data.append(b2)
        self.cmdstate += 1

    def handle_status(self, mosi, miso):
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
            self.on_end_transaction = lambda: self.putc([Ann.STATUS, [cmds[self.state][1]]])
        else:
            # Will be overwritten for each byte.
            self.es_cmd = self.es
            self.es_field = self.es
            if self.cmdstate == 2:
                self.ss_field = self.ss
            self.putx([Ann.BIT, ['Status register byte %d: 0x%02x' % ((self.cmdstate % 2) + 1, miso)]])
        self.cmdstate += 1

    # TODO: Warn/abort if we don't see the necessary amount of bytes.
    def handle_se(self, mosi, miso):
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
            if self.writestate == 0:
                self.putx([Ann.WARN, ['Warning: WREN might be missing']])
        elif self.cmdstate in (2, 3, 4):
            # Bytes 2/3/4: Master sends sector address (24bits, MSB-first).
            self.emit_addr_bytes(mosi)

        if self.cmdstate == 4:
            self.es_cmd = self.es
            d = 'Erase sector %d (0x%06x)' % (self.addr, self.addr)
            self.putc([Ann.SE, [d]])
            # TODO: Max. size depends on chip, check that too if possible.
            if self.addr % 4096 != 0:
                # Sector addresses must be 4K-aligned (same for all 3 chips).
                self.putc([Ann.WARN, ['Warning: Invalid sector address!']])
            self.state = None
        else:
            self.cmdstate += 1

    def handle_be(self, mosi, miso):
        pass # TODO

    def handle_ce(self, mosi, miso):
        self.putx([Ann.CE, self.cmd_ann_list()])
        if self.writestate == 0:
            self.putx([Ann.WARN, ['Warning: WREN might be missing']])

    def handle_ce2(self, mosi, miso):
        self.putx([Ann.CE2, self.cmd_ann_list()])
        if self.writestate == 0:
            self.putx([Ann.WARN, ['Warning: WREN might be missing']])

    def handle_pp(self, mosi, miso):
        # Page program: Master asserts CS#, sends PP command, sends 3-byte
        # page address, sends >= 1 data bytes, de-asserts CS#.
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate in (2, 3, 4):
            # Bytes 2/3/4: Master sends page address (24bits, MSB-first).
            self.emit_addr_bytes(mosi)
        elif self.cmdstate >= 5:
            # Bytes 5-x: Master sends data bytes (until CS# de-asserted).
            self.es_field = self.es # Will be overwritten for each byte.
            if self.cmdstate == 5:
                self.ss_field = self.ss
                self.on_end_transaction = lambda: self.output_data_block('Data', Ann.PP)
            self.data.append(mosi)
        self.cmdstate += 1

    def handle_cp(self, mosi, miso):
        pass # TODO

    def handle_dp(self, mosi, miso):
        pass # TODO

    def handle_rdp_res(self, mosi, miso):
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate in (2, 3, 4):
            # Bytes 2/3/4: Master sends three dummy bytes.
            self.putx([Ann.FIELD, ['Dummy byte: %02x' % mosi]])
        elif self.cmdstate == 5:
            # Byte 5: Slave sends device ID.
            self.es_cmd = self.es
            self.device_id = miso
            self.putx([Ann.FIELD, ['Device ID: %s' % self.device()]])
            d = 'Device = %s' % self.vendor_device()
            self.putc([Ann.RDP_RES, self.cmd_vendor_dev_list()])
            self.state = None
        self.cmdstate += 1

    def handle_rems(self, mosi, miso):
        if self.cmdstate == 1:
            # Byte 1: Master sends command ID.
            self.emit_cmd_byte()
        elif self.cmdstate in (2, 3):
            # Bytes 2/3: Master sends two dummy bytes.
            self.putx([Ann.FIELD, ['Dummy byte: 0x%02x' % mosi]])
        elif self.cmdstate == 4:
            # Byte 4: Master sends 0x00 or 0x01.
            # 0x00: Master wants manufacturer ID as first reply byte.
            # 0x01: Master wants device ID as first reply byte.
            self.manufacturer_id_first = True if (mosi == 0x00) else False
            d = 'manufacturer' if (mosi == 0x00) else 'device'
            self.putx([Ann.FIELD, ['Master wants %s ID first' % d]])
        elif self.cmdstate == 5:
            # Byte 5: Slave sends manufacturer ID (or device ID).
            self.ids = [miso]
            d = 'Manufacturer' if self.manufacturer_id_first else 'Device'
            self.putx([Ann.FIELD, ['%s ID: 0x%02x' % (d, miso)]])
        elif self.cmdstate == 6:
            # Byte 6: Slave sends device ID (or manufacturer ID).
            self.ids.append(miso)
            d = 'Device' if self.manufacturer_id_first else 'Manufacturer'
            self.putx([Ann.FIELD, ['%s ID: 0x%02x' % (d, miso)]])

        if self.cmdstate == 6:
            id_ = self.ids[1] if self.manufacturer_id_first else self.ids[0]
            self.device_id = id_
            self.es_cmd = self.es
            self.putc([Ann.REMS, self.cmd_vendor_dev_list()])
            self.state = None
        else:
            self.cmdstate += 1

    def handle_rems2(self, mosi, miso):
        pass # TODO

    def handle_enso(self, mosi, miso):
        pass # TODO

    def handle_exso(self, mosi, miso):
        pass # TODO

    def handle_rdscur(self, mosi, miso):
        pass # TODO

    def handle_wrscur(self, mosi, miso):
        pass # TODO

    def handle_esry(self, mosi, miso):
        pass # TODO

    def handle_dsry(self, mosi, miso):
        pass # TODO

    def output_data_block(self, label, idx):
        # Print accumulated block of data
        # (called on CS# de-assert via self.on_end_transaction callback).
        self.es_cmd = self.es # End on the CS# de-assert sample.
        if self.options['format'] == 'hex':
            s = ' '.join([('%02x' % b) for b in self.data])
        else:
            s = ''.join(map(chr, self.data))
        self.putf([Ann.FIELD, ['%s (%d bytes)' % (label, len(self.data))]])
        self.putc([idx, ['%s (addr 0x%06x, %d bytes): %s' % \
                   (cmds[self.state][1], self.addr, len(self.data), s)]])

    def decode(self, ss, es, data):
        ptype, mosi, miso = data

        self.ss, self.es = ss, es

        if ptype == 'CS-CHANGE':
            self.end_current_transaction()

        if ptype != 'DATA':
            return

        # If we encountered a known chip command, enter the resp. state.
        if self.state is None:
            self.state = mosi
            self.cmdstate = 1

        # Handle commands.
        try:
            self.cmd_handlers[self.state](mosi, miso)
        except KeyError:
            self.putx([Ann.BIT, ['Unknown command: 0x%02x' % mosi]])
            self.state = None



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2010-2016 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# TODO: Look into arbitration, collision detection, clock synchronisation, etc.
# TODO: Implement support for inverting SDA/SCL levels (0->1 and 1->0).
# TODO: Implement support for detecting various bus errors.

import sigrokdecode as srd

'''
OUTPUT_PYTHON format:

Packet:
[<ptype>, <pdata>]

<ptype>:
 - 'START' (START condition)
 - 'START REPEAT' (Repeated START condition)
 - 'ADDRESS READ' (Slave address, read)
 - 'ADDRESS WRITE' (Slave address, write)
 - 'DATA READ' (Data, read)
 - 'DATA WRITE' (Data, write)
 - 'STOP' (STOP condition)
 - 'ACK' (ACK bit)
 - 'NACK' (NACK bit)
 - 'BITS' (<pdata>: list of data/address bits and their ss/es numbers)

<pdata> is the data or address byte associated with the 'ADDRESS*' and 'DATA*'
command. Slave addresses do not include bit 0 (the READ/WRITE indication bit).
For example, a slave address field could be 0x51 (instead of 0xa2).
For 'START', 'START REPEAT', 'STOP', 'ACK', and 'NACK' <pdata> is None.
'''

# CMD: [annotation-type-index, long annotation, short annotation]
proto = {
    'START':           [0, 'Start',         'S'],
    'START REPEAT':    [1, 'Start repeat',  'Sr'],
    'STOP':            [2, 'Stop',          'P'],
    'ACK':             [3, 'ACK',           'A'],
    'NACK':            [4, 'NACK',          'N'],
    'BIT':             [5, 'Bit',           'B'],
    'ADDRESS READ':    [6, 'Address read',  'AR'],
    'ADDRESS WRITE':   [7, 'Address write', 'AW'],
    'DATA READ':       [8, 'Data read',     'DR'],
    'DATA WRITE':      [9, 'Data write',    'DW'],
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'i2c'
    name = 'I²C'
    longname = 'Inter-Integrated Circuit'
    desc = 'Two-wire, multi-master, serial bus.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = ['i2c']
    tags = ['Embedded/industrial']
    channels = (
        {'id': 'scl', 'name': 'SCL', 'desc': 'Serial clock line'},
        {'id': 'sda', 'name': 'SDA', 'desc': 'Serial data line'},
    )
    options = (
        {'id': 'address_format', 'desc': 'Displayed slave address format',
            'default': 'shifted', 'values': ('shifted', 'unshifted')},
    )
    annotations = (
        ('start', 'Start condition'),
        ('repeat-start', 'Repeat start condition'),
        ('stop', 'Stop condition'),
        ('ack', 'ACK'),
        ('nack', 'NACK'),
        ('bit', 'Data/address bit'),
        ('address-read', 'Address read'),
        ('address-write', 'Address write'),
        ('data-read', 'Data read'),
        ('data-write', 'Data write'),
        ('warnings', 'Human-readable warnings'),
    )
    annotation_rows = (
        ('bits', 'Bits', (5,)),
        ('addr-data', 'Address/Data', (0, 1, 2, 3, 4, 6, 7, 8, 9)),
        ('warnings', 'Warnings', (10,)),
    )
    binary = (
        ('address-read', 'Address read'),
        ('address-write', 'Address write'),
        ('data-read', 'Data read'),
        ('data-write', 'Data write'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.ss = self.es = self.ss_byte = -1
        self.bitcount = 0
        self.databyte = 0
        self.wr = -1
        self.is_repeat_start = 0
        self.state = 'FIND START'
        self.pdu_start = None
        self.pdu_bits = 0
        self.bits = []

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def start(self):
        self.out_python = self.register(srd.OUTPUT_PYTHON)
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.out_bitrate = self.register(srd.OUTPUT_META,
                meta=(int, 'Bitrate', 'Bitrate from Start bit to Stop bit'))

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def putp(self, data):
        self.put(self.ss, self.es, self.out_python, data)

    def putb(self, data):
        self.put(self.ss, self.es, self.out_binary, data)

    def handle_start(self, pins):
        self.ss, self.es = self.samplenum, self.samplenum
        self.pdu_start = self.samplenum
        self.pdu_bits = 0
        cmd = 'START REPEAT' if (self.is_repeat_start == 1) else 'START'
        self.putp([cmd, None])
        self.putx([proto[cmd][0], proto[cmd][1:]])
        self.state = 'FIND ADDRESS'
        self.bitcount = self.databyte = 0
        self.is_repeat_start = 1
        self.wr = -1
        self.bits = []

    # Gather 8 bits of data plus the ACK/NACK bit.
    def handle_address_or_data(self, pins):
        scl, sda = pins
        self.pdu_bits += 1

        # Address and data are transmitted MSB-first.
        self.databyte <<= 1
        self.databyte |= sda

        # Remember the start of the first data/address bit.
        if self.bitcount == 0:
            self.ss_byte = self.samplenum

        # Store individual bits and their start/end samplenumbers.
        # In the list, index 0 represents the LSB (I²C transmits MSB-first).
        self.bits.insert(0, [sda, self.samplenum, self.samplenum])
        if self.bitcount > 0:
            self.bits[1][2] = self.samplenum
        if self.bitcount == 7:
            self.bitwidth = self.bits[1][2] - self.bits[2][2]
            self.bits[0][2] += self.bitwidth

        # Return if we haven't collected all 8 + 1 bits, yet.
        if self.bitcount < 7:
            self.bitcount += 1
            return

        d = self.databyte
        if self.state == 'FIND ADDRESS':
            # The READ/WRITE bit is only in address bytes, not data bytes.
            self.wr = 0 if (self.databyte & 1) else 1
            if self.options['address_format'] == 'shifted':
                d = d >> 1

        bin_class = -1
        if self.state == 'FIND ADDRESS' and self.wr == 1:
            cmd = 'ADDRESS WRITE'
            bin_class = 1
        elif self.state == 'FIND ADDRESS' and self.wr == 0:
            cmd = 'ADDRESS READ'
            bin_class = 0
        elif self.state == 'FIND DATA' and self.wr == 1:
            cmd = 'DATA WRITE'
            bin_class = 3
        elif self.state == 'FIND DATA' and self.wr == 0:
            cmd = 'DATA READ'
            bin_class = 2

        self.ss, self.es = self.ss_byte, self.samplenum + self.bitwidth

        self.putp(['BITS', self.bits])
        self.putp([cmd, d])

        self.putb([bin_class, bytes([d])])

        for bit in self.bits:
            self.put(bit[1], bit[2], self.out_ann, [5, ['%d' % bit[0]]])

        if cmd.startswith('ADDRESS'):
            self.ss, self.es = self.samplenum, self.samplenum + self.bitwidth
            w = ['Write', 'Wr', 'W'] if self.wr else ['Read', 'Rd', 'R']
            self.putx([proto[cmd][0], w])
            self.ss, self.es = self.ss_byte, self.samplenum

        self.putx([proto[cmd][0], ['%s: %02X' % (proto[cmd][1], d),
                   '%s: %02X' % (proto[cmd][2], d), '%02X' % d]])

        # Done with this packet.
        self.bitcount = self.databyte = 0
        self.bits = []
        self.state = 'FIND ACK'

    def get_ack(self, pins):
        scl, sda = pins
        self.ss, self.es = self.samplenum, self.samplenum + self.bitwidth
        cmd = 'NACK' if (sda == 1) else 'ACK'
        self.putp([cmd, None])
        self.putx([proto[cmd][0], proto[cmd][1:]])
        # There could be multiple data bytes in a row, so either find
        # another data byte or a STOP condition next.
        self.state = 'FIND DATA'

    def handle_stop(self, pins):
        # Meta bitrate
        if self.samplerate:
            elapsed = 1 / float(self.samplerate) * (self.samplenum - self.pdu_start + 1)
            bitrate = int(1 / elapsed * self.pdu_bits)
            self.put(self.ss_byte, self.samplenum, self.out_bitrate, bitrate)

        cmd = 'STOP'
        self.ss, self.es = self.samplenum, self.samplenum
        self.putp([cmd, None])
        self.putx([proto[cmd][0], proto[cmd][1:]])
        self.state = 'FIND START'
        self.is_repeat_start = 0
        self.wr = -1
        self.bits = []

    def decode(self):
        while True:
            # State machine.
            if self.state == 'FIND START':
                # Wait for a START condition (S): SCL = high, SDA = falling.
                self.handle_start(self.wait({0: 'h', 1: 'f'}))
            elif self.state == 'FIND ADDRESS':
                # Wait for a data bit: SCL = rising.
                self.handle_address_or_data(self.wait({0: 'r'}))
            elif self.state == 'FIND DATA':
                # Wait for any of the following conditions (or combinations):
                #  a) Data sampling of receiver: SCL = rising, and/or
                #  b) START condition (S): SCL = high, SDA = falling, and/or
                #  c) STOP condition (P): SCL = high, SDA = rising
                pins = self.wait([{0: 'r'}, {0: 'h', 1: 'f'}, {0: 'h', 1: 'r'}])

                # Check which of the condition(s) matched and handle them.
                if self.matched[0]:
                    self.handle_address_or_data(pins)
                elif self.matched[1]:
                    self.handle_start(pins)
                elif self.matched[2]:
                    self.handle_stop(pins)
            elif self.state == 'FIND ACK':
                # Wait for a data/ack bit: SCL = rising.
                self.get_ack(self.wait({0: 'r'}))



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012-2015 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

dacs = {
    0: 'DACA',
    1: 'DACB',
    2: 'DACC',
    3: 'DACD',
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'tlc5620'
    name = 'TI TLC5620'
    longname = 'Texas Instruments TLC5620'
    desc = 'Texas Instruments TLC5620 8-bit quad DAC.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['IC', 'Analog/digital']
    channels = (
        {'id': 'clk', 'name': 'CLK', 'desc': 'Serial interface clock'},
        {'id': 'data', 'name': 'DATA', 'desc': 'Serial interface data'},
    )
    optional_channels = (
        {'id': 'load', 'name': 'LOAD', 'desc': 'Serial interface load control'},
        {'id': 'ldac', 'name': 'LDAC', 'desc': 'Load DAC'},
    )
    options = (
        {'id': 'vref_a', 'desc': 'Reference voltage DACA (V)', 'default': 3.3},
        {'id': 'vref_b', 'desc': 'Reference voltage DACB (V)', 'default': 3.3},
        {'id': 'vref_c', 'desc': 'Reference voltage DACC (V)', 'default': 3.3},
        {'id': 'vref_d', 'desc': 'Reference voltage DACD (V)', 'default': 3.3},
    )
    annotations = (
        ('dac-select', 'DAC select'),
        ('gain', 'Gain'),
        ('value', 'DAC value'),
        ('data-latch', 'Data latch point'),
        ('ldac-fall', 'LDAC falling edge'),
        ('bit', 'Bit'),
        ('reg-write', 'Register write'),
        ('voltage-update', 'Voltage update'),
        ('voltage-update-all', 'Voltage update (all DACs)'),
        ('invalid-cmd', 'Invalid command'),
    )
    annotation_rows = (
        ('bits', 'Bits', (5,)),
        ('fields', 'Fields', (0, 1, 2)),
        ('registers', 'Registers', (6, 7)),
        ('voltage-updates', 'Voltage updates', (8,)),
        ('events', 'Events', (3, 4)),
        ('errors', 'Errors', (9,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.bits = []
        self.ss_dac_first = None
        self.ss_dac = self.es_dac = 0
        self.ss_gain = self.es_gain = 0
        self.ss_value = self.es_value = 0
        self.dac_select = self.gain = self.dac_value = None
        self.dacval = {'A': '?', 'B': '?', 'C': '?', 'D': '?'}
        self.gains = {'A': '?', 'B': '?', 'C': '?', 'D': '?'}

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def handle_11bits(self):
        # Only look at the last 11 bits, the rest is ignored by the TLC5620.
        if len(self.bits) > 11:
            self.bits = self.bits[-11:]

        # If there are less than 11 bits, something is probably wrong.
        if len(self.bits) < 11:
            ss, es = self.samplenum, self.samplenum
            if len(self.bits) >= 2:
                ss = self.bits[0][1]
                es = self.bits[-1][1] + (self.bits[1][1] - self.bits[0][1])
            self.put(ss, es, self.out_ann, [9, ['Command too short']])
            self.bits = []
            return False

        self.ss_dac = self.bits[0][1]
        self.es_dac = self.ss_gain = self.bits[2][1]
        self.es_gain = self.ss_value = self.bits[3][1]
        self.clock_width = self.es_gain - self.ss_gain
        self.es_value = self.bits[10][1] + self.clock_width # Guessed.

        if self.ss_dac_first is None:
            self.ss_dac_first = self.ss_dac

        s = ''.join(str(i[0]) for i in self.bits[:2])
        self.dac_select = s = dacs[int(s, 2)]
        self.put(self.ss_dac, self.es_dac, self.out_ann,
                 [0, ['DAC select: %s' % s, 'DAC sel: %s' % s,
                      'DAC: %s' % s, 'D: %s' % s, s, s[3]]])

        self.gain = g = 1 + self.bits[2][0]
        self.put(self.ss_gain, self.es_gain, self.out_ann,
                 [1, ['Gain: x%d' % g, 'G: x%d' % g, 'x%d' % g]])

        s = ''.join(str(i[0]) for i in self.bits[3:])
        self.dac_value = v = int(s, 2)
        self.put(self.ss_value, self.es_value, self.out_ann,
                 [2, ['DAC value: %d' % v, 'Value: %d' % v, 'Val: %d' % v,
                      'V: %d' % v, '%d' % v]])

        # Emit an annotation for each bit.
        for i in range(1, 11):
            self.put(self.bits[i - 1][1], self.bits[i][1], self.out_ann,
                     [5, [str(self.bits[i - 1][0])]])
        self.put(self.bits[10][1], self.bits[10][1] + self.clock_width,
                 self.out_ann, [5, [str(self.bits[10][0])]])

        self.bits = []

        return True

    def handle_falling_edge_load(self):
        if not self.handle_11bits():
            return
        s, v, g = self.dac_select, self.dac_value, self.gain
        self.put(self.samplenum, self.samplenum, self.out_ann,
                 [3, ['Falling edge on LOAD', 'LOAD fall', 'F']])
        vref = self.options['vref_%s' % self.dac_select[3].lower()]
        v = '%.2fV' % (vref * (v / 256) * self.gain)
        if self.ldac == 0:
            # If LDAC is low, the voltage is set immediately.
            self.put(self.ss_dac, self.es_value, self.out_ann,
                     [7, ['Setting %s voltage to %s' % (s, v),
                          '%s=%s' % (s, v)]])
        else:
            # If LDAC is high, the voltage is not set immediately, but rather
            # stored in a register. When LDAC goes low all four DAC voltages
            # (DAC A/B/C/D) will be set at the same time.
            self.put(self.ss_dac, self.es_value, self.out_ann,
                     [6, ['Setting %s register value to %s' % \
                          (s, v), '%s=%s' % (s, v)]])
        # Save the last value the respective DAC was set to.
        self.dacval[self.dac_select[-1]] = str(self.dac_value)
        self.gains[self.dac_select[-1]] = self.gain

    def handle_falling_edge_ldac(self):
        self.put(self.samplenum, self.samplenum, self.out_ann,
                 [4, ['Falling edge on LDAC', 'LDAC fall', 'LDAC', 'L']])

        # Don't emit any annotations if we didn't see any register writes.
        if self.ss_dac_first is None:
            return

        # Calculate voltages based on Vref and the per-DAC gain.
        dacval = {}
        for key, val in self.dacval.items():
            if val == '?':
                dacval[key] = '?'
            else:
                vref = self.options['vref_%s' % key.lower()]
                v = vref * (int(val) / 256) * self.gains[key]
                dacval[key] = '%.2fV' % v

        s = ''.join(['DAC%s=%s ' % (d, dacval[d]) for d in 'ABCD']).strip()
        self.put(self.ss_dac_first, self.samplenum, self.out_ann,
                 [8, ['Updating voltages: %s' % s, s, s.replace('DAC', '')]])
        self.ss_dac_first = None

    def handle_new_dac_bit(self, datapin):
        self.bits.append([datapin, self.samplenum])

    def decode(self):
        while True:
            # DATA is shifted in the DAC on the falling CLK edge (MSB-first).
            # A falling edge of LOAD will latch the data.

            # Wait for one (or multiple) of the following conditions:
            #   a) Falling edge on CLK, and/or
            #   b) Falling edge on LOAD, and/or
            #   b) Falling edge on LDAC
            pins = self.wait([{0: 'f'}, {2: 'f'}, {3: 'f'}])
            self.ldac = pins[3]

            # Handle those conditions (one or more) that matched this time.
            if self.matched[0]:
                self.handle_new_dac_bit(pins[1])
            if self.matched[1]:
                self.handle_falling_edge_load()
            if self.matched[2]:
                self.handle_falling_edge_ldac()



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Steve R <steversig@virginmedia.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

# Most of the info here comes from "434MHz RF Protocol Descriptions for
# Wireless Weather Sensors - October 2015" Known Sensor ID Codes - p25.

# Format is 4 hex digit ID code followed by a LIST of models that use that
# ID and the type of sensor.
# SensorID is used as the hash in a Python hash table, so it must be upper case.
# The type of sensor is used to decode and display readings in the L2 decode,
# it's case-sensitive.
# Be very careful with the formatting ' [] and commas.

sensor = {
#   'SensorID': [['model1', 'model2'], 'type'],
    '1984': [['WGR800'], 'Wind'], # The newer anemometer with no temperature/RH sensor.
    '1994': [['WGR800'], 'Wind'], # The original anemometer which included a temperature/RH sensor.
    '1A2D': [['THGR228N'], 'Temp_Hum1'],
    '1A3D': [['THGR918'], ''],
    '1D20': [['THGN123N', 'THGR122NX', 'THGN123N', 'THGR228N'], 'Temp_Hum'],
    '1D30': [['THGN500', 'THGN132N'], ''],
    '2914': [['PCR800'], 'Rain'],
    '2A19': [['PCR800'], 'Rain1'],
    '2A1D': [['RGR918'], 'Rain'],
    '2D10': [['RGR968', 'PGR968 '], 'Rain1'],
    '3A0D': [['STR918', 'WGR918'], 'Wind'],
    '5A5D': [['BTHR918'], ''],
    '5A6D': [['BTHR918N'], 'Temp_Hum_Baro'],
    '5D53': [['BTHGN129'], 'Baro'],
    '5D60': [['BTHR968'], 'Temp_Hum_Baro'],
    'C844': [['THWR800'], 'Temp'],
    'CC13': [['RTGR328N'], 'Temp_Hum'],
    'CC23': [['THGR328N'], 'Temp_Hum'],
    'CD39': [['RTHR328N'], 'Temp'],
    'D874': [['UVN800'], 'UV1'],
    'EA4C': [['THWR288A'], 'Temp'],
    'EC40': [['THN132N', 'THR238NF'], 'Temp'],
    'EC70': [['UVR128'], 'UV'],
    'F824': [['THGN800', 'THGN801', 'THGR810'], 'Temp_Hum'],
    'F8B4': [['THGR810'], 'Temp_Hum'],
#    '': ['PSR01'], '', ''],
#    '': ['RTGR328NA'], '', ''],
#    '': ['THC268'], '', ''],
#    '': ['THWR288A-JD'], '', ''],
#    '': ['THGR268'], '', ''],
#    '': ['THR268'], '', ''],
}

# The sensor checksum exceptions are used to calculate the right checksum for
# sensors that don't follow the v1, v2.1 and v3 methods. For instance a v2.1
# sensor that has a v3 checksum.
sensor_checksum = {
#   'SensorID': ['checksum_method', 'comment'],
    '1D20': ['v3', 'THGR228N'],
    '5D60': ['v3', 'BTHR918N'],
    'EC40': ['v3', 'THN132N'],
}

dir_table = ['N', 'NNE', 'NE', 'ENE', 'E', 'ESE', 'SE', 'SSE', 'S', 'SSW', 'SW', 'WSW', 'W', 'WNW', 'NW', 'NNW', 'N']



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Karl Palsson <karlp@etactica.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
This decoder stacks on top of the 'spi' PD and decodes Analog Devices
ADE77xx command/responses.

The ADE77xx is a "Poly Phase Multifunction Energy Metering IC with Per Phase
Information".

This PD has been tested with an ADE7758 so far, support for other devices
from the ADE77xx series can be added in the future.
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2019 Stephan Thiele <stephan.thiele@mailbox.org>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

# Selection of constants as defined in FlexRay specification 3.0.1 Chapter A.1:
class Const:
    cChannelIdleDelimiter = 11
    cCrcInitA = 0xFEDCBA
    cCrcInitB = 0xABCDEF
    cCrcPolynomial = 0x5D6DCB
    cCrcSize = 24
    cCycleCountMax = 63
    cdBSS = 2
    cdCAS = 30
    cdFES = 2
    cdFSS = 1
    cHCrcInit = 0x01A
    cHCrcPolynomial = 0x385
    cHCrcSize = 11
    cSamplesPerBit = 8
    cSlotIDMax = 2047
    cStaticSlotIDMax = 1023
    cVotingSamples = 5

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'flexray'
    name = 'FlexRay'
    longname = 'FlexRay'
    desc = 'Automotive network communications protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Automotive']
    channels = (
        {'id': 'channel', 'name': 'Channel', 'desc': 'FlexRay bus channel'},
    )
    options = (
        {'id': 'channel_type', 'desc': 'Channel type', 'default': 'A',
            'values': ('A', 'B')},
        {'id': 'bitrate', 'desc': 'Bitrate (bit/s)', 'default': 10000000,
            'values': (10000000, 5000000, 2500000)},
    )
    annotations = (
        ('data', 'FlexRay payload data'),
        ('tss', 'Transmission start sequence'),
        ('fss', 'Frame start sequence'),
        ('reserved-bit', 'Reserved bit'),
        ('ppi', 'Payload preamble indicator'),
        ('null-frame', 'Nullframe indicator'),
        ('sync-frame', 'Full identifier'),
        ('startup-frame', 'Startup frame indicator'),
        ('id', 'Frame ID'),
        ('length', 'Data length'),
        ('header-crc', 'Header CRC'),
        ('cycle', 'Cycle code'),
        ('data-byte', 'Data byte'),
        ('frame-crc', 'Frame CRC'),
        ('cid-delimiter', 'Channel idle delimiter'),
        ('bss', 'Byte start sequence'),
        ('warnings', 'Human-readable warnings'),
        ('bit', 'Bit'),
        ('cid', 'Channel idle delimiter'),
        ('dts', 'Dynamic trailing sequence'),
        ('cas', 'Collision avoidance symbol'),
    )
    annotation_rows = (
        ('bits', 'Bits', (15, 17)),
        ('fields', 'Fields', tuple(range(15)) + (18, 19, 20)),
        ('warnings', 'Warnings', (16,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.reset_variables()

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            bitrate = float(self.options['bitrate'])
            self.samplerate = value
            self.bit_width = float(self.samplerate) / bitrate
            self.sample_point = (self.bit_width / 100.0) * self.sample_point_percent

    # Generic helper for FlexRay bit annotations.
    def putg(self, ss, es, data):
        left, right = int(self.sample_point), int(self.bit_width - self.sample_point)
        self.put(ss - left, es + right, self.out_ann, data)

    # Single-FlexRay-bit annotation using the current samplenum.
    def putx(self, data):
        self.putg(self.samplenum, self.samplenum, data)

    # Multi-FlexRay-bit annotation from self.ss_block to current samplenum.
    def putb(self, data):
        self.putg(self.ss_block, self.samplenum, data)

    # Generic CRC algorithm for any bit size and any data length. Used for
    # 11-bit header and 24-bit trailer. Not very efficient but at least it
    # works for now.
    #
    # TODO:
    # - use precalculated tables to increase performance.
    # - Add support for reverse CRC calculations.

    @staticmethod
    def crc(data, data_len_bits, polynom, crc_len_bits, iv=0, xor=0):
        reg = iv ^ xor

        for i in range(data_len_bits - 1, -1, -1):
            bit = ((reg >> (crc_len_bits - 1)) & 0x1) ^ ((data >> i) & 0x1)
            reg <<= 1
            if bit:
                reg ^= polynom

        mask = (1 << crc_len_bits) - 1
        crc = reg & mask

        return crc ^ xor

    def reset_variables(self):
        self.sample_point_percent = 50 # TODO: use vote based sampling
        self.state = 'IDLE'
        self.tss_start = self.tss_end = self.frame_type = self.dlc = None
        self.rawbits = [] # All bits, including byte start sequence bits
        self.bits = [] # Only actual FlexRay frame bits (no byte start sequence bits)
        self.curbit = 0 # Current bit of FlexRay frame (bit 0 == FSS)
        self.last_databit = 999 # Positive value that bitnum+x will never match
        self.last_xmit_bit = 999 # Positive value that bitnum+x will never match
        self.ss_block = None
        self.ss_databytebits = []
        self.end_of_frame = False
        self.dynamic_frame = False
        self.ss_bit0 = None
        self.ss_bit1 = None
        self.ss_bit2 = None

    # Poor man's clock synchronization. Use signal edges which change to
    # dominant state in rather simple ways. This naive approach is neither
    # aware of the SYNC phase's width nor the specific location of the edge,
    # but improves the decoder's reliability when the input signal's bitrate
    # does not exactly match the nominal rate.
    def dom_edge_seen(self, force=False):
        self.dom_edge_snum = self.samplenum
        self.dom_edge_bcount = self.curbit

    # Determine the position of the next desired bit's sample point.
    def get_sample_point(self, bitnum):
        samplenum = self.dom_edge_snum
        samplenum += self.bit_width * (bitnum - self.dom_edge_bcount)
        samplenum += self.sample_point
        return int(samplenum)

    def is_bss_sequence(self):
        # FlexRay uses NRZ encoding and adds a binary 10 sequence before each
        # byte. After each 8 data bits, a BSS sequence is added but not after
        # frame CRC.

        if self.end_of_frame:
          return False

        if (len(self.rawbits) - 2) % 10 == 0:
            return True
        elif (len(self.rawbits) - 3) % 10 == 0:
            return True

        return False

    def handle_bit(self, fr_rx):
        self.rawbits.append(fr_rx)
        self.bits.append(fr_rx)

        # Get the index of the current FlexRay frame bit.
        bitnum = len(self.bits) - 1

        # If this is a byte start sequence remove it from self.bits and ignore it.
        if self.is_bss_sequence():
            self.bits.pop()

            if bitnum > 1:
                self.putx([15, [str(fr_rx)]])
            else:
                if len(self.rawbits) == 2:
                    self.ss_bit1 = self.samplenum
                elif len(self.rawbits) == 3:
                    self.ss_bit2 = self.samplenum

            self.curbit += 1 # Increase self.curbit (bitnum is not affected).
            return
        else:
            if bitnum > 1:
                self.putx([17, [str(fr_rx)]])

        # Bit 0: Frame start sequence (FSS) bit
        if bitnum == 0:
            self.ss_bit0 = self.samplenum

        # Bit 1: Start of header
        elif bitnum == 1:
            if self.rawbits[:3] == [1, 1, 0]:
                self.put(self.tss_start, self.tss_end, self.out_ann,
                        [1, ['Transmission start sequence', 'TSS']])

                self.putg(self.ss_bit0, self.ss_bit0, [17, [str(self.rawbits[:3][0])]])
                self.putg(self.ss_bit0, self.ss_bit0, [2,  ['FSS', 'Frame start sequence']])
                self.putg(self.ss_bit1, self.ss_bit1, [15, [str(self.rawbits[:3][1])]])
                self.putg(self.ss_bit2, self.ss_bit2, [15, [str(self.rawbits[:3][2])]])
                self.putx([17, [str(fr_rx)]])
                self.putx([3, ['Reserved bit: %d' % fr_rx, 'RB: %d' % fr_rx, 'RB']])
            else:
                self.put(self.tss_start, self.tss_end, self.out_ann,
                        [20, ['Collision avoidance symbol', 'CAS']])
                self.reset_variables()

            # TODO: warning, if sequence is neither [1, 1, 0] nor [1, 1, 1]

        # Bit 2: Payload preamble indicator. Must be 0 if null frame indicator is 0.
        elif bitnum == 2:
            self.putx([4, ['Payload preamble indicator: %d' % fr_rx,
                           'PPI: %d' % fr_rx]])

        # Bit 3: Null frame indicator (inversed)
        elif bitnum == 3:
            data_type = 'data frame' if fr_rx else 'null frame'
            self.putx([5, ['Null frame indicator: %s' % data_type,
                           'NF: %d' % fr_rx, 'NF']])

        # Bit 4: Sync frame indicator
        # Must be 1 if startup frame indicator is 1.
        elif bitnum == 4:
            self.putx([6, ['Sync frame indicator: %d' % fr_rx,
                           'Sync: %d' % fr_rx, 'Sync']])

        # Bit 5: Startup frame indicator
        elif bitnum == 5:
            self.putx([7, ['Startup frame indicator: %d' % fr_rx,
                           'Startup: %d' % fr_rx, 'Startup']])

        # Remember start of ID (see below).
        elif bitnum == 6:
            self.ss_block = self.samplenum

        # Bits 6-16: Frame identifier (ID[10..0])
        # ID must NOT be 0.
        elif bitnum == 16:
            self.id = int(''.join(str(d) for d in self.bits[6:]), 2)
            self.putb([8, ['Frame ID: %d' % self.id, 'ID: %d' % self.id,
                           '%d' % self.id]])

        # Remember start of payload length (see below).
        elif bitnum == 17:
            self.ss_block = self.samplenum

        # Bits 17-23: Payload length (Length[7..0])
        # Payload length in header is the half of the real payload size.
        elif bitnum == 23:
            self.payload_length = int(''.join(str(d) for d in self.bits[17:]), 2)
            self.putb([9, ['Payload length: %d' % self.payload_length,
                           'Length: %d' % self.payload_length,
                           '%d' % self.payload_length]])

        # Remember start of header CRC (see below).
        elif bitnum == 24:
            self.ss_block = self.samplenum

        # Bits 24-34: Header CRC (11-bit) (HCRC[11..0])
        # Calculation of header CRC is equal on both channels.
        elif bitnum == 34:
            bits = ''.join([str(b) for b in self.bits[4:24]])
            header_to_check = int(bits, 2)
            expected_crc = self.crc(header_to_check, len(bits),
                Const.cHCrcPolynomial, Const.cHCrcSize, Const.cHCrcInit)
            self.header_crc = int(''.join(str(d) for d in self.bits[24:]), 2)

            crc_ok = self.header_crc == expected_crc
            crc_ann = "OK" if crc_ok else "bad"

            self.putb([10, ['Header CRC: 0x%X (%s)' % (self.header_crc, crc_ann),
                            '0x%X (%s)' % (self.header_crc, crc_ann),
                            '0x%X' % self.header_crc]])

        # Remember start of cycle code (see below).
        elif bitnum == 35:
            self.ss_block = self.samplenum

        # Bits 35-40: Cycle code (Cyc[6..0])
        # Cycle code. Must be between 0 and 63.
        elif bitnum == 40:
            self.cycle = int(''.join(str(d) for d in self.bits[35:]), 2)
            self.putb([11, ['Cycle: %d' % self.cycle, 'Cyc: %d' % self.cycle,
                            '%d' % self.cycle]])
            self.last_databit = 41 + 2 * self.payload_length * 8

        # Remember all databyte bits, except the very last one.
        elif bitnum in range(41, self.last_databit):
            self.ss_databytebits.append(self.samplenum)

        # Bits 41-X: Data field (0-254 bytes, depending on length)
        # The bits within a data byte are transferred MSB-first.
        elif bitnum == self.last_databit:
            self.ss_databytebits.append(self.samplenum) # Last databyte bit.
            for i in range(2 * self.payload_length):
                x = 40 + (8 * i) + 1
                b = int(''.join(str(d) for d in self.bits[x:x + 8]), 2)
                ss = self.ss_databytebits[i * 8]
                es = self.ss_databytebits[((i + 1) * 8) - 1]
                self.putg(ss, es, [12, ['Data byte %d: 0x%02x' % (i, b),
                                        'DB%d: 0x%02x' % (i, b), '%02X' % b]])
            self.ss_databytebits = []
            self.ss_block = self.samplenum # Remember start of trailer CRC.

        # Trailer CRC (24-bit) (CRC[11..0])
        # Initialization vector of channel A and B are different, so CRCs are
        # different for same data.
        elif bitnum == self.last_databit + 23:
            bits = ''.join([str(b) for b in self.bits[1:-24]])
            frame_to_check = int(bits, 2)
            iv = Const.cCrcInitA if self.options['channel_type'] == 'A' else Const.cCrcInitB
            expected_crc = self.crc(frame_to_check, len(bits),
                Const.cCrcPolynomial, Const.cCrcSize, iv=iv)
            self.frame_crc = int(''.join(str(d) for d in self.bits[self.last_databit:]), 2)

            crc_ok = self.frame_crc == expected_crc
            crc_ann = "OK" if crc_ok else "bad"

            self.putb([13, ['Frame CRC: 0x%X (%s)' % (self.frame_crc, crc_ann),
                            '0x%X (%s)' % (self.frame_crc, crc_ann),
                            '0x%X' % self.frame_crc]])
            self.end_of_frame = True

        # Remember start of frame end sequence (see below).
        elif bitnum == self.last_databit + 24:
            self.ss_block = self.samplenum

        # Frame end sequence, must be 1 followed by 0.
        elif bitnum == self.last_databit + 25:
            self.putb([14, ['Frame end sequence', 'FES']])

        # Check for DTS
        elif bitnum == self.last_databit + 26:
            if not fr_rx:
                self.dynamic_frame = True
            else:
                self.last_xmit_bit = bitnum
            self.ss_block = self.samplenum

        # Remember start of channel idle delimiter (see below).
        elif bitnum == self.last_xmit_bit:
            self.ss_block = self.samplenum

        # Channel idle limiter (CID[11..0])
        elif bitnum == self.last_xmit_bit + Const.cChannelIdleDelimiter - 1:
            self.putb([18, ['Channel idle delimiter', 'CID']])
            self.reset_variables()

        # DTS if dynamic frame
        elif bitnum > self.last_databit + 27:
            if self.dynamic_frame:
                if fr_rx:
                    if self.last_xmit_bit == 999:
                        self.putb([19, ['Dynamic trailing sequence', 'DTS']])
                        self.last_xmit_bit = bitnum + 1
                        self.ss_block = self.samplenum

        self.curbit += 1

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')

        while True:
            # State machine.
            if self.state == 'IDLE':
                # Wait for a dominant state (logic 0) on the bus.
                (fr_rx,) = self.wait({0: 'l'})
                self.tss_start = self.samplenum
                (fr_rx,) = self.wait({0: 'h'})
                self.tss_end = self.samplenum
                self.dom_edge_seen(force = True)
                self.state = 'GET BITS'
            elif self.state == 'GET BITS':
                # Wait until we're in the correct bit/sampling position.
                pos = self.get_sample_point(self.curbit)
                (fr_rx,) = self.wait([{'skip': pos - self.samplenum}, {0: 'f'}])
                if self.matched[1]:
                    self.dom_edge_seen()
                if self.matched[0]:
                    self.handle_bit(fr_rx)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Dave Craig <dcraig@brightsign.biz>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

msg_ids = {
    2: 'AKE_Init',
    3: 'AKE_Send_Cert',
    4: 'AKE_No_stored_km',
    5: 'AKE_Stored_km',

    7: 'AKE_Send_H_prime',
    8: 'AKE_Send_Pairing_Info',

    9: 'LC_Init',
    10: 'LC_Send_L_prime',

    11: 'SKE_Send_Eks',
    12: 'RepeaterAuth_Send_ReceiverID_List',

    15: 'RepeaterAuth_Send_Ack',
    16: 'RepeaterAuth_Stream_Manage',
    17: 'RepeaterAuth_Stream_Ready',
}

write_items = {
    0x00: '1.4 Bksv - Receiver KSV',
    0x08: '1.4 Ri\' - Link Verification',
    0x0a: '1.4 Pj\' - Enhanced Link Verification',
    0x10: '1.4 Aksv - Transmitter KSV',
    0x15: '1.4 Ainfo - Transmitter KSV',
    0x18: '1.4 An - Session random number',
    0x20: '1.4 V\'H0',
    0x24: '1.4 V\'H1',
    0x28: '1.4 V\'H2',
    0x2c: '1.4 V\'H3',
    0x30: '1.4 V\'H4',
    0x40: '1.4 Bcaps',
    0x41: '1.4 Bstatus',
    0x43: '1.4 KSV FIFO',
    0x50: 'HDCP2Version',
    0x60: 'Write_Message',
    0x70: 'RxStatus',
    0x80: 'Read_Message',
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'hdcp'
    name = 'HDCP'
    longname = 'HDCP over HDMI'
    desc = 'HDCP protocol over HDMI.'
    license = 'gplv2+'
    inputs = ['i2c']
    outputs = ['hdcp']
    tags = ['PC', 'Security/crypto']
    annotations = \
        tuple(('message-0x%02X' % i, 'Message 0x%02X' % i) for i in range(18)) + (
        ('summary', 'Summary'),
        ('warnings', 'Warnings'),
    )
    annotation_rows = (
        ('messages', 'Messages', tuple(range(18))),
        ('summary', 'Summary', (18,)),
        ('warnings', 'Warnings', (19,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IDLE'
        self.stack = []
        self.msg = -1
        self.ss = self.es = self.ss_block = self.es_block = 0
        self.init_seq = []
        self.valid = 0
        self.type = ''

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putb(self, data):
        self.put(self.ss_block, self.es_block, self.out_ann, data)

    def decode(self, ss, es, data):
        cmd, databyte = data

        # Collect the 'BITS' packet, then return. The next packet is
        # guaranteed to belong to these bits we just stored.
        if cmd == 'BITS':
            self.bits = databyte
            return

        self.ss, self.es = ss, es

        # State machine.
        if self.state == 'IDLE':
            # Wait for an I2C START condition.
            if cmd == 'START':
                self.reset()
                self.ss_block = ss
            elif cmd != 'START REPEAT':
                return
            self.state = 'GET SLAVE ADDR'
        elif self.state == 'GET SLAVE ADDR':
            if cmd == 'ADDRESS READ':
                self.state = 'BUFFER DATA'
                if databyte != 0x3a:
                    self.state = 'IDLE'
            elif cmd == 'ADDRESS WRITE':
                self.state = 'WRITE OFFSET'
                if databyte != 0x3a:
                    self.state = 'IDLE'
        elif self.state == 'WRITE OFFSET':
            if cmd == 'DATA WRITE':
                if databyte in write_items:
                    self.type = write_items[databyte]
                if databyte in (0x10, 0x15, 0x18, 0x60):
                    self.state = 'BUFFER DATA'
                # If we are reading, then jump back to IDLE for a start repeat.
                # If we are writing, then just continue onwards.
                if self.state == 'BUFFER DATA':
                    pass
                elif self.type != '':
                    self.state = 'IDLE'
        elif self.state == 'BUFFER DATA':
            if cmd in ('STOP', 'NACK'):
                self.es_block = es
                self.state = 'IDLE'
                if self.type == '':
                    return
                if not self.stack:
                    self.putb([18, ['%s' % (self.type)]])
                    return
                if self.type == 'RxStatus':
                    rxstatus = (self.stack.pop() << 8) | self.stack.pop()
                    reauth_req = (rxstatus & 0x800) != 0
                    ready = (rxstatus & 0x400) != 0
                    length = rxstatus & 0x3ff
                    text = '%s, reauth %s, ready %s, length %s' % \
                        (self.type, reauth_req, ready, length)
                    self.putb([18, [text]])
                elif self.type == '1.4 Bstatus':
                    bstatus = (self.stack.pop() << 8) | self.stack.pop()
                    device_count = bstatus & 0x7f
                    max_devs_exceeded = (bstatus & 0x80) != 0
                    depth = ((bstatus & 0x700) >> 8)
                    max_cascase_exceeded = bstatus & 0x800
                    hdmi_mode = (bstatus & 0x1000) != 0
                    text = '%s, %s devices, depth %s, hdmi mode %s' % \
                        (self.type, device_count, depth, hdmi_mode)
                    self.putb([18, [text]])
                elif self.type == 'Read_Message':
                    msg = self.stack.pop(0)
                    self.putb([msg, ['%s, %s' % (self.type,
                        msg_ids.get(msg, 'Invalid'))]])
                elif self.type == 'Write_Message':
                    msg = self.stack.pop(0)
                    self.putb([msg, ['%s, %s' % (self.type,
                        msg_ids.get(msg, 'Invalid'))]])
                elif self.type == 'HDCP2Version':
                    version = self.stack.pop(0)
                    if (version & 0x4):
                        self.putb([18, ['HDCP2']])
                    else:
                        self.putb([18, ['NOT HDCP2']])
                else:
                    self.putb([18, ['%s' % (self.type)]])
            elif cmd == 'DATA READ':
                # Stack up our data bytes.
                self.stack.append(databyte)
            elif cmd == 'DATA WRITE':
                # Stack up our data bytes.
                self.stack.append(databyte)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2019 Vesa-Pekka Palmu <vpalmu@depili.fi>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

from collections import OrderedDict

# OrderedDict which maps command IDs to their names and descriptions.
cmds = OrderedDict([
    (0xA0, ('PAGE', 'Jump to page')),
    (0xD0, ('GBV', 'Get byte variable')),
    (0xD1, ('GWV', 'Get word variable')),
    (0xD2, ('GSV', 'Get string variable')),
    (0xD3, ('GLV', 'Get label variable')),
    (0xD4, ('GRPC', 'Get RPC buffer')),
    (0xD5, ('SBV', 'Set byte variable')),
    (0xD6, ('SWV', 'Set word variable')),
    (0xD7, ('SSV', 'Set string variable')),
    (0xD8, ('RPC', 'Invoke RPC')),
    (0xD9, ('LINE', 'Draw line')),
    (0xDA, ('RECT', 'Draw rectangle')),
    (0xDB, ('FRECT', 'Draw filled rectangle')),
    (0xDC, ('PIXEL', 'Draw pixel')),
    (0xDD, ('GBVA', 'Get byte variable array')),
    (0xDE, ('GWVA', 'Get word variable array')),
    (0xDF, ('SBVA', 'Set byte variable array')),
    (0xE0, ('GBVR', 'Get byte variable reply')),
    (0xE1, ('GWVR', 'Get word variable reply')),
    (0xE2, ('GSVR', 'Get string variable reply')),
    (0xE3, ('GLVR', 'Get label variable reply')),
    (0xE4, ('GRPCR', 'Get RPC buffer reply')),
    (0xE5, ('SBVR', 'Set byte variable reply')),
    (0xE6, ('SWVR', 'Set word variable reply')),
    (0xE7, ('SSVR', 'Set string variable reply')),
    (0xE8, ('RPCR', 'Invoke RPC reply')),
    (0xE9, ('LINER', 'Draw line reply')),
    (0xEA, ('RECTR', 'Draw rectangle')),
    (0xEB, ('FRECTR', 'Draw filled rectangle reply')),
    (0xEC, ('PIXELR', 'Draw pixel reply')),
    (0xED, ('GBVAR', 'Get byte variable array reply')),
    (0xEE, ('GWVAR', 'Get word variable array reply')),
    (0xEF, ('SBVAR', 'Set byte variable array reply')),
    (0xF0, ('ACK', 'Acknowledgment')),
    (0xF1, ('NACK', 'Negative acknowledgment')),
    (0xF2, ('SWVA', 'Set word variable array')),
    (0xF3, ('SWVAR', 'Set word variable array reply')),
    (0xF4, ('GCV', 'Get color variable')),
    (0xF5, ('GCVR', 'Get color variable reply')),
    (0xF6, ('SCV', 'Set color variable')),
    (0xF7, ('SCVR', 'Set color variable reply')),
])

cmds_with_high_bytes = [
    0xA0, # PAGE - Page change
    0xD7, # SVV - Set string variable
    0xE7, # SVVR - Set string variable reply
    0xE2, # GSVR - Get string variable reply
    0xE3, # GLVR - Get label variable reply
]
